@@ -18,15 +18,11 @@
  */
 
 import cockpit from "cockpit";
-import { mustache } from "mustache";
-import day_header_template from 'raw-loader!journal_day_header.mustache';
-import line_template from 'raw-loader!journal_line.mustache';
-import reboot_template from 'raw-loader!journal_reboot.mustache';
+import * as timeformat from "timeformat";
 
 const _ = cockpit.gettext;
-var C_ = cockpit.gettext;
 
-export var journal = { };
+export const journal = { };
 
 /**
  * journalctl([match, ...], [options])
@@ -54,6 +50,7 @@ export var journal = { };
  *  * "until": if specified list entries until the date/time
  *  * "cursor": a cursor to start listing entries from
  *  * "after": a cursor to start listing entries after
+ *  * "priority": if specified list entries below the specific priority, inclusive
  *
  * Returns a jQuery deferred promise. You can call these
  * functions on the deferred to handle the responses. Note that
@@ -62,7 +59,7 @@ export var journal = { };
  *  .done(function(entries) { }): Called when done, @entries is
  *         an array of all journal entries loaded. If .stream()
  *         has been invoked then @entries will be empty.
- *  .fail(funciton(ex) { }): called if the operation fails
+ *  .fail(function(ex) { }): called if the operation fails
  *  .stream(function(entries) { }): called when we receive entries
  *         entries. Called once per batch of journal @entries,
  *         whether following or not.
@@ -70,18 +67,17 @@ export var journal = { };
  */
 
 journal.build_cmd = function build_cmd(/* ... */) {
-    var matches = [];
-    var i, arg;
-    var options = { follow: true };
-    for (i = 0; i < arguments.length; i++) {
-        arg = arguments[i];
+    const matches = [];
+    const options = { follow: true };
+    for (let i = 0; i < arguments.length; i++) {
+        const arg = arguments[i];
         if (typeof arg == "string") {
             matches.push(arg);
         } else if (typeof arg == "object") {
             if (arg instanceof Array) {
                 matches.push.apply(matches, arg);
             } else {
-                cockpit.extend(options, arg);
+                Object.assign(options, arg);
                 break;
             }
         } else {
@@ -96,7 +92,7 @@ journal.build_cmd = function build_cmd(/* ... */) {
             options.count = null;
     }
 
-    var cmd = ["journalctl", "-q"];
+    const cmd = ["journalctl", "-q"];
     if (!options.count)
         cmd.push("--no-tail");
     else
@@ -118,6 +114,10 @@ journal.build_cmd = function build_cmd(/* ... */) {
         cmd.push("--cursor=" + options.cursor);
     if (options.after)
         cmd.push("--after=" + options.after);
+    if (options.priority)
+        cmd.push("--priority=" + options.priority);
+    if (options.grep)
+        cmd.push("--grep=" + options.grep);
 
     /* journalctl doesn't allow reverse and follow together */
     if (options.reverse)
@@ -131,17 +131,17 @@ journal.build_cmd = function build_cmd(/* ... */) {
 };
 
 journal.journalctl = function journalctl(/* ... */) {
-    var [cmd, options] = journal.build_cmd.apply(null, arguments);
+    const [cmd, options] = journal.build_cmd.apply(null, arguments);
 
-    var dfd = cockpit.defer();
-    var promise;
-    var buffer = "";
-    var entries = [];
-    var streamers = [];
-    var interval = null;
+    const dfd = cockpit.defer();
+    const promise = dfd.promise();
+    let buffer = "";
+    let entries = [];
+    let streamers = [];
+    let interval = null;
 
     function fire_streamers() {
-        var ents, i;
+        let ents, i;
         if (streamers.length && entries.length > 0) {
             ents = entries;
             entries = [];
@@ -153,14 +153,14 @@ journal.journalctl = function journalctl(/* ... */) {
         }
     }
 
-    var proc = cockpit.spawn(cmd, { host: options.host, batch: 8192, latency: 300, superuser: "try" })
+    const proc = cockpit.spawn(cmd, { host: options.host, batch: 8192, latency: 300, superuser: "try" })
             .stream(function(data) {
                 if (buffer)
                     data = buffer + data;
                 buffer = "";
 
-                var lines = data.split("\n");
-                var last = lines.length - 1;
+                const lines = data.split("\n");
+                const last = lines.length - 1;
                 lines.forEach(function(line, i) {
                     if (i == last) {
                         buffer = line;
@@ -195,12 +195,12 @@ journal.journalctl = function journalctl(/* ... */) {
                 window.clearInterval(interval);
             });
 
-    promise = dfd.promise();
     promise.stream = function stream(callback) {
         streamers.push(callback);
         return this;
     };
     promise.stop = function stop() {
+        streamers = [];
         proc.close("cancelled");
     };
     return promise;
@@ -217,94 +217,7 @@ journal.printable = function printable(value) {
         return _("[binary data]");
 };
 
-function output_funcs_for_box(box) {
-    /* Dereference any jQuery object here */
-    if (box.jquery)
-        box = box[0];
-
-    mustache.parse(day_header_template);
-    mustache.parse(line_template);
-    mustache.parse(reboot_template);
-
-    function render_line(ident, prio, message, count, time, entry) {
-        var parts = {
-            cursor: entry.__CURSOR,
-            time: time,
-            message: message,
-            service: ident
-        };
-        if (count > 1)
-            parts.count = count;
-        if (ident === 'abrt-notification') {
-            parts.problem = true;
-            parts.service = entry.PROBLEM_BINARY;
-        } else if (prio < 4)
-            parts.warning = true;
-        return mustache.render(line_template, parts);
-    }
-
-    var reboot = _("Reboot");
-    var reboot_line = mustache.render(reboot_template, { message: reboot });
-
-    function render_reboot_separator() {
-        return reboot_line;
-    }
-
-    function render_day_header(day) {
-        return mustache.render(day_header_template, { day: day });
-    }
-
-    function parse_html(string) {
-        var div = document.createElement("div");
-        div.innerHTML = string.trim();
-        return div.children[0];
-    }
-
-    return {
-        render_line: render_line,
-        render_day_header: render_day_header,
-        render_reboot_separator: render_reboot_separator,
-
-        append: function(elt) {
-            if (typeof (elt) == "string")
-                elt = parse_html(elt);
-            box.appendChild(elt);
-        },
-        prepend: function(elt) {
-            if (typeof (elt) == "string")
-                elt = parse_html(elt);
-            if (box.firstChild)
-                box.insertBefore(elt, box.firstChild);
-            else
-                box.appendChild(elt);
-        },
-        remove_last: function() {
-            if (box.lastChild)
-                box.removeChild(box.lastChild);
-        },
-        remove_first: function() {
-            if (box.firstChild)
-                box.removeChild(box.firstChild);
-        },
-    };
-}
-
-var month_names = [
-    C_("month-name", 'January'),
-    C_("month-name", 'February'),
-    C_("month-name", 'March'),
-    C_("month-name", 'April'),
-    C_("month-name", 'May'),
-    C_("month-name", 'June'),
-    C_("month-name", 'July'),
-    C_("month-name", 'August'),
-    C_("month-name", 'September'),
-    C_("month-name", 'October'),
-    C_("month-name", 'November'),
-    C_("month-name", 'December')
-];
-
-/* Render the journal entries by passing suitable HTML strings back to
+/* Render the journal entries by passing suitable DOM elements back to
    the caller via the 'output_funcs'.
 
    Rendering is context aware.  It will insert 'reboot' markers, for
@@ -314,7 +227,7 @@ var month_names = [
    A new renderer is created by calling 'journal.renderer' like
    so:
 
-      var renderer = journal.renderer(funcs);
+      const renderer = journal.renderer(funcs);
 
    You can feed new entries into the renderer by calling various
    methods on the returned object:
@@ -364,18 +277,14 @@ var month_names = [
       - output_funcs.render_line(ident, prio, message, count, time, cursor)
       - output_funcs.render_day_header(day)
       - output_funcs.render_reboot_separator()
-
 */
 
-journal.renderer = function renderer(funcs_or_box) {
-    var output_funcs;
-    if (funcs_or_box.render_line)
-        output_funcs = funcs_or_box;
-    else
-        output_funcs = output_funcs_for_box(funcs_or_box);
+journal.renderer = function renderer(output_funcs) {
+    if (!output_funcs.render_line)
+        console.error("Invalid renderer provided");
 
     function copy_object(o) {
-        var c = { }; for (var p in o) c[p] = o[p]; return c;
+        const c = { }; for (const p in o) c[p] = o[p]; return c;
     }
 
     // A 'entry' object describes a journal entry in formatted form.
@@ -383,19 +292,12 @@ journal.renderer = function renderer(funcs_or_box) {
     // 'day', all of which are strings.
 
     function format_entry(journal_entry) {
-        function pad(n) {
-            var str = n.toFixed();
-            if (str.length == 1)
-                str = '0' + str;
-            return str;
-        }
-
-        var d = new Date(journal_entry.__REALTIME_TIMESTAMP / 1000);
+        const d = journal_entry.__REALTIME_TIMESTAMP / 1000; // timestamps are in µs
         return {
             cursor: journal_entry.__CURSOR,
             full: journal_entry,
-            day: month_names[d.getMonth()] + ' ' + d.getDate().toFixed() + ', ' + d.getFullYear().toFixed(),
-            time: pad(d.getHours()) + ':' + pad(d.getMinutes()),
+            day: timeformat.date(d),
+            time: timeformat.time(d),
             bootid: journal_entry._BOOT_ID,
             ident: journal_entry.SYSLOG_IDENTIFIER || journal_entry._COMM,
             prio: journal_entry.PRIORITY,
@@ -444,7 +346,7 @@ journal.renderer = function renderer(funcs_or_box) {
     // line has been output so far, top_state and bottom_state point
     // to the same object.
 
-    var top_state, bottom_state;
+    let top_state, bottom_state;
 
     top_state = bottom_state = { };
 
@@ -471,7 +373,7 @@ journal.renderer = function renderer(funcs_or_box) {
     }
 
     function prepend(journal_entry) {
-        var entry = format_entry(journal_entry);
+        const entry = format_entry(journal_entry);
 
         if (entry_is_equal(top_state.entry, entry)) {
             top_state.count += 1;
@@ -514,7 +416,7 @@ journal.renderer = function renderer(funcs_or_box) {
     }
 
     function append(journal_entry) {
-        var entry = format_entry(journal_entry);
+        const entry = format_entry(journal_entry);
 
         if (entry_is_equal(bottom_state.entry, entry)) {
             bottom_state.count += 1;
@@ -542,47 +444,9 @@ journal.renderer = function renderer(funcs_or_box) {
     }
 
     return {
-        prepend: prepend,
-        prepend_flush: prepend_flush,
-        append: append,
-        append_flush: append_flush
+        prepend,
+        prepend_flush,
+        append,
+        append_flush
     };
 };
-
-journal.logbox = function logbox(match, max_entries) {
-    var entries = [];
-    var box = document.createElement("div");
-
-    function render() {
-        var renderer = journal.renderer(box);
-        while (box.firstChild)
-            box.removeChild(box.firstChild);
-        for (var i = 0; i < entries.length; i++) {
-            renderer.prepend(entries[i]);
-        }
-        renderer.prepend_flush();
-        if (entries.length === 0) {
-            const empty_message = document.createElement("span");
-            empty_message.textContent = "No log entries";
-            empty_message.setAttribute("class", "empty-message");
-            box.appendChild(empty_message);
-        }
-    }
-
-    render();
-
-    var promise = journal.journalctl(match, { count: max_entries })
-            .stream(function(tail) {
-                entries = entries.concat(tail);
-                if (entries.length > max_entries)
-                    entries = entries.slice(-max_entries);
-                render();
-            })
-            .fail(function(error) {
-                box.appendChild(document.createTextNode(error.message));
-                box.removeAttribute("hidden");
-            });
-
-    /* Both a DOM element and a promise */
-    return promise.promise(box);
-};
