@@ -20,9 +20,10 @@
 import cockpit from 'cockpit';
 import * as service from 'service';
 import { debounce } from 'throttle-debounce';
-import * as utils from './utils';
+import * as utils from './utils.js';
 
-var firewall = {
+const firewall = {
+    owner: undefined,
     installed: true,
     enabled: false,
     readonly: true,
@@ -37,6 +38,7 @@ var firewall = {
         'dmz', 'work', 'home', 'internal', 'trusted'],
     defaultZone: null,
     availableInterfaces: [],
+    ready: false
 };
 
 cockpit.event_target(firewall);
@@ -46,7 +48,7 @@ utils.list_interfaces().then(interfaces => {
 });
 
 const firewalld_service = service.proxy('firewalld');
-var firewalld_dbus = null;
+let firewalld_dbus = null;
 
 firewall.debouncedGetZones = debounce(300, () => {
     getZones()
@@ -70,6 +72,10 @@ function initFirewalldDbus() {
     firewalld_dbus = cockpit.dbus('org.fedoraproject.FirewallD1', { superuser: "try" });
 
     firewalld_dbus.addEventListener('owner', (event, owner) => {
+        if (firewall.owner === owner)
+            return;
+
+        firewall.owner = owner;
         firewall.enabled = !!owner;
 
         firewall.zones = {};
@@ -78,14 +84,16 @@ function initFirewalldDbus() {
         firewall.enabledServices = new Set();
 
         if (!firewall.enabled) {
+            firewall.ready = true;
             firewall.dispatchEvent('changed');
             return;
         }
 
         getZones()
                 .then(() => getServices())
-                .then(() => firewall.debouncedEvent('changed'))
-                .catch(error => console.warn(error));
+                .catch(error => console.warn(error))
+                .then(() => { firewall.ready = true })
+                .then(() => firewall.debouncedEvent('changed'));
     });
 
     firewalld_dbus.subscribe({
@@ -115,6 +123,33 @@ function initFirewalldDbus() {
         firewall.debouncedGetServices();
     });
 
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'PortAdded'
+    }, (path, iface, signal, args) => {
+        const zone = args[0];
+        const port = args[1];
+        const protocol = args[2];
+        if (!firewall.zones[zone].ports.some(p => p.port === port && p.protocol === protocol)) {
+            firewall.zones[zone].ports.push({ port, protocol });
+            firewall.debouncedEvent('changed');
+        }
+    });
+
+    firewalld_dbus.subscribe({
+        interface: 'org.fedoraproject.FirewallD1.zone',
+        path: '/org/fedoraproject/FirewallD1',
+        member: 'PortRemoved'
+    }, (path, iface, signal, args) => {
+        const zone = args[0];
+        const port = args[1];
+        const protocol = args[2];
+        firewall.zones[zone].ports = firewall.zones[zone].ports
+                .filter(p => p.port !== port || p.protocol !== protocol);
+        firewall.debouncedEvent('changed');
+    });
+
     firewalld_dbus.subscribe({
         interface: 'org.fedoraproject.FirewallD1',
         path: '/org/fedoraproject/FirewallD1',
@@ -221,15 +256,15 @@ function fetchServiceInfos(services) {
         let info;
         return firewalld_dbus.call('/org/fedoraproject/FirewallD1',
                                    'org.fedoraproject.FirewallD1',
-                                   'getServiceSettings', [service])
-                .then(reply => {
-                    const name = reply[0][1];
-                    const description = reply[0][2];
-                    const ports = reply[0][3];
+                                   'getServiceSettings2', [service])
+                .then(([{ short, description, ports }]) => {
+                    short = short ? short.v : "";
+                    description = description ? description.v : "";
+                    ports = ports ? ports.v : [];
                     info = {
                         id: service,
-                        name: name,
-                        description: description,
+                        name: short,
+                        description,
                         ports: ports.map(p => ({ port: p[0], protocol: p[1] })),
                         includes: [],
                     };
@@ -252,7 +287,7 @@ function fetchServiceInfos(services) {
                 .catch(error => {
                     if (error.name === 'org.freedesktop.DBus.Error.UnknownMethod')
                         return info;
-                    Promise.reject(error);
+                    return Promise.reject(error);
                 });
     }));
 }
@@ -274,19 +309,18 @@ function fetchZoneInfos(zones) {
             return info;
         }
         return firewalld_dbus.call('/org/fedoraproject/FirewallD1',
-                                   'org.fedoraproject.FirewallD1',
-                                   'getZoneSettings', [zone])
-                .then(reply => {
-                    const [, name, description, , target, services, ports, , , , interfaces, source] = reply[0];
+                                   'org.fedoraproject.FirewallD1.zone',
+                                   'getZoneSettings2', [zone])
+                .then(([zoneInfo]) => {
                     const info = {
                         id: zone,
-                        name: name,
-                        description: description,
-                        target: target,
-                        services: services,
-                        ports: ports.map(p => ({ port: p[0], protocol: p[1] })),
-                        interfaces: interfaces,
-                        source: source,
+                        name: (zoneInfo.short || {}).v,
+                        description: (zoneInfo.description || {}).v,
+                        target: (zoneInfo.target || {}).v,
+                        services: ((zoneInfo.services || {}).v || []),
+                        ports: ((zoneInfo.ports || {}).v || []).map(p => ({ port: p[0], protocol: p[1] })),
+                        interfaces: ((zoneInfo.interfaces || {}).v || []),
+                        source: ((zoneInfo.sources || {}).v || []),
                     };
                     firewall.zones[zone] = info;
                     return info;
@@ -296,15 +330,28 @@ function fetchZoneInfos(zones) {
 
 initFirewalldDbus();
 
-cockpit.spawn(['sh', '-c', 'pkcheck --action-id org.fedoraproject.FirewallD1.all --process $$ --allow-user-interaction 2>&1'])
-        .done(() => {
+cockpit.spawn(['sh', '-c', 'pkcheck --action-id org.fedoraproject.FirewallD1.all --process $$ --allow-user-interaction 2>&1'], { superuser: "try" })
+        .then(() => {
             firewall.readonly = false;
+            firewall.debouncedEvent('changed');
             firewall.debouncedGetZones();
+        })
+        .catch(error => {
+            console.log("pkcheck failed", error);
+
+            // Fall back to cockpit.permissions, "pkcheck" might not be available,
+            // always allow edits by admins
+            const permission = cockpit.permission({ admin: true });
+            const update_permissions = () => {
+                firewall.readonly = !permission.allowed;
+                firewall.debouncedEvent('changed');
+                firewall.debouncedGetZones();
+            };
+            permission.addEventListener("changed", update_permissions);
         });
 
-firewall.enable = () => Promise.all([firewalld_service.enable(), firewalld_service.start()]);
-
-firewall.disable = () => Promise.all([firewalld_service.stop(), firewalld_service.disable()]);
+firewall.enable = () => firewalld_service.enable().then(() => firewalld_service.start());
+firewall.disable = () => firewalld_service.disable().then(() => firewalld_service.stop());
 
 firewall.getAvailableServices = () => {
     return firewalld_dbus.call('/org/fedoraproject/FirewallD1',
@@ -347,7 +394,7 @@ firewall.removeService = (zone, service) => {
  * Returns a promise that resolves when the service is created.
  * It will also reload firewalld and enable the new service.
  */
-firewall.createService = (service, name, ports, zones) => {
+firewall.createService = (service, ports, zones, desc = "") => {
     const subscription = firewalld_dbus.subscribe({
         interface: 'org.fedoraproject.FirewallD1',
         path: '/org/fedoraproject/FirewallD1',
@@ -358,10 +405,32 @@ firewall.createService = (service, name, ports, zones) => {
     });
     return firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
                                'org.fedoraproject.FirewallD1.config',
-                               'addService', [service, ["", name, "", ports, [], {}, [], []]])
+                               'addService2', [service, { description: { t: 's', v: desc }, ports: { t: 'a(ss)', v: ports } }])
             .then(() => firewall.reload());
 };
 
+/*
+ * Edit firewalld service.
+ *
+ * Returns a promise that resolves when the service is edited.
+ */
+firewall.editService = (service, ports, desc = "") => {
+    return firewalld_dbus.call('/org/fedoraproject/FirewallD1/config',
+                               'org.fedoraproject.FirewallD1.config',
+                               'getServiceByName', [service])
+            .then(path => firewalld_dbus.call(path[0],
+                                              'org.fedoraproject.FirewallD1.config.service',
+                                              'update2', [{ description: { t: 's', v: desc }, ports: { t: 'a(ss)', v: ports } }])
+                    .then(() => {
+                        // No signal for updated service so we need to manually update it
+                        firewall.services[service].description = desc;
+                        firewall.services[service].ports = ports.map(port => ({ port: port[0], protocol: port[1] }));
+                        firewall.debouncedEvent('changed');
+                        return firewall.reload();
+                    })
+            );
+};
+
 /*
  * Add a predefined firewalld service to the specified zone (i.e., open its
  * ports).
@@ -416,14 +485,7 @@ firewall.activateZone = (zone, interfaces, sources) => {
 
         return firewalld_dbus.call(path[0],
                                    'org.fedoraproject.FirewallD1.config.zone',
-                                   'getSettings', [])
-                .then(settings => {
-                    settings[0][10] = interfaces;
-                    settings[0][11] = sources;
-                    return firewalld_dbus.call(path[0],
-                                               'org.fedoraproject.FirewallD1.config.zone',
-                                               'update', [settings[0]]);
-                });
+                                   'update2', [{ interfaces: { t: 'as', v: interfaces }, sources: { t: 'as', v: sources } }]);
     });
     return p;
 };
@@ -457,14 +519,7 @@ firewall.deactiveateZone = (zone) => {
 
         return firewalld_dbus.call(path[0],
                                    'org.fedoraproject.FirewallD1.config.zone',
-                                   'getSettings', [])
-                .then(settings => {
-                    settings[0][10] = [];
-                    settings[0][11] = [];
-                    return firewalld_dbus.call(path[0],
-                                               'org.fedoraproject.FirewallD1.config.zone',
-                                               'update', [settings[0]]);
-                });
+                                   'update2', [{ interfaces: { t: 'as', v: [] }, sources: { t: 'as', v: [] } }]);
     });
 
     return p.catch(error => console.warn(error));
