@@ -17,40 +17,44 @@
  * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
  */
 
-import $ from "jquery";
 import cockpit from "cockpit";
-import { showDialog } from "./active-pages";
+import React from "react";
+import { createRoot } from "react-dom/client";
 
-var shell_embedded = window.location.pathname.indexOf(".html") !== -1;
+import { TimeoutModal } from "./shell-modals.jsx";
+
+const shell_embedded = window.location.pathname.indexOf(".html") !== -1;
 const _ = cockpit.gettext;
 
 function component_checksum(machine, component) {
-    var parts = component.split("/");
-    var pkg = parts[0];
+    const parts = component.split("/");
+    const pkg = parts[0];
     if (machine.manifests && machine.manifests[pkg] && machine.manifests[pkg][".checksum"])
         return "$" + machine.manifests[pkg][".checksum"];
 }
 
-function Frames(index) {
-    var self = this;
+function Frames(index, setupIdleResetTimers) {
+    const self = this;
+    let language = document.cookie.replace(/(?:(?:^|.*;\s*)CockpitLang\s*=\s*([^;]*).*$)|^.*$/, "$1");
+    if (!language)
+        language = navigator.language.toLowerCase(); // Default to Accept-Language header
 
     /* Lists of frames, by host */
     self.iframes = { };
 
     function remove_frame(frame) {
-        $(frame.contentWindow).off();
-        $(frame).remove();
+        frame.remove();
     }
 
     self.remove = function remove(machine, component) {
-        var address;
+        let address;
         if (typeof machine == "string")
             address = machine;
         else if (machine)
             address = machine.address;
         if (!address)
             address = "localhost";
-        var list = self.iframes[address] || { };
+        const list = self.iframes[address] || { };
         if (!component)
             delete self.iframes[address];
         Object.keys(list).forEach(function(key) {
@@ -62,13 +66,14 @@ function Frames(index) {
     };
 
     function frame_ready(frame, count) {
-        var ready = false;
+        let ready = false;
 
         window.clearTimeout(frame.timer);
         frame.timer = null;
 
         try {
-            ready = $("body", frame.contentWindow.document).is(":visible");
+            if (frame.contentWindow.document && frame.contentWindow.document.body)
+                ready = frame.contentWindow.document.body.offsetWidth > 0 && frame.contentWindow.document.body.offsetHeight > 0;
         } catch (ex) {
             ready = true;
         }
@@ -85,6 +90,14 @@ function Frames(index) {
                 if (count > 0)
                     index.navigate();
             }
+            if (frame.contentWindow && setupIdleResetTimers)
+                setupIdleResetTimers(frame.contentWindow);
+
+            if (frame.contentDocument && frame.contentDocument.documentElement) {
+                frame.contentDocument.documentElement.lang = language;
+                if (cockpit.language_direction)
+                    frame.contentDocument.documentElement.dir = cockpit.language_direction;
+            }
         } else {
             frame.timer = window.setTimeout(function() {
                 frame_ready(frame, count + 1);
@@ -93,9 +106,9 @@ function Frames(index) {
     }
 
     self.lookup = function lookup(machine, component, hash) {
-        var host;
-        var address;
-        var new_frame = false;
+        let host;
+        let address;
+        let new_frame = false;
 
         if (typeof machine == "string") {
             address = host = machine;
@@ -109,26 +122,24 @@ function Frames(index) {
         if (!address)
             address = host;
 
-        var list = self.iframes[address];
+        let list = self.iframes[address];
         if (!list)
             self.iframes[address] = list = { };
 
-        var name = "cockpit1:" + host + "/" + component;
-        var frame = list[component];
+        const name = "cockpit1:" + host + "/" + component;
+        let frame = list[component];
         if (frame && frame.getAttribute("name") != name) {
             remove_frame(frame);
             frame = null;
         }
 
-        var wind, src;
-
         /* A preloaded frame */
         if (!frame) {
-            wind = window.frames[name];
+            const wind = window.frames[name];
             if (wind)
                 frame = wind.frameElement;
             if (frame) {
-                src = frame.getAttribute('src');
+                const src = frame.getAttribute('src');
                 frame.url = src.split("#")[0];
                 list[component] = frame;
             }
@@ -140,9 +151,10 @@ function Frames(index) {
             frame = document.createElement("iframe");
             frame.setAttribute("class", "container-frame");
             frame.setAttribute("name", name);
+            frame.setAttribute("data-host", host);
             frame.style.display = "none";
 
-            var base, checksum;
+            let base, checksum;
             if (machine) {
                 if (machine.manifests && machine.manifests[".checksum"])
                     checksum = "$" + machine.manifests[".checksum"];
@@ -177,14 +189,44 @@ function Frames(index) {
 
         if (!hash)
             hash = "/";
-        src = frame.url + "#" + hash;
-        if (frame.getAttribute('src') != src)
+        const src = frame.url + "#" + hash;
+        if (frame.getAttribute('src') != src) {
+            if (frame.contentWindow) {
+                // This prevents the browser from creating a new
+                // history entry.  It would do that whenever the "src"
+                // of a frame is changed and the window location is
+                // not consistent with the new "src" value.
+                //
+                // This matters when a "jump" command changes both the
+                // the current frame and the hash of the new frame.
+                frame.contentWindow.location.replace(src);
+            }
             frame.setAttribute('src', src);
+        }
 
         /* Store frame only when fully setup */
         if (new_frame) {
             list[component] = frame;
-            $("#content").append(frame);
+            document.getElementById("content").appendChild(frame);
+
+            const style = localStorage.getItem('shell:style') || 'auto';
+            let dark_mode;
+            // If a user set's an explicit theme, ignore system changes.
+            if ((window.matchMedia && window.matchMedia('(prefers-color-scheme: dark)').matches && style === "auto") || style === "dark") {
+                dark_mode = true;
+            } else {
+                dark_mode = false;
+            }
+
+            // The new iframe is shown before any HTML/CSS is ready and loaded,
+            // explicitly set a dark background so we don't see any white flashes
+            if (dark_mode && frame.contentDocument && frame.contentDocument.documentElement) {
+                // --pf-global--BackgroundColor--dark-300
+                const dark_mode_background = '#1b1d21';
+                frame.contentDocument.documentElement.style.background = dark_mode_background;
+            } else {
+                frame.contentDocument.documentElement.style.background = 'white';
+            }
         }
         frame_ready(frame);
         return frame;
@@ -192,37 +234,34 @@ function Frames(index) {
 }
 
 function Router(index) {
-    var self = this;
+    const self = this;
 
-    var unique_id = 0;
-    var origin = cockpit.transport.origin;
-    var source_by_seed = { };
-    var source_by_name = { };
+    let unique_id = 0;
+    const origin = cockpit.transport.origin;
+    const source_by_seed = { };
+    const source_by_name = { };
 
     cockpit.transport.filter(function(message, channel, control) {
-        var seed, source, pos;
-
         /* Only control messages with a channel are forwardable */
         if (control) {
             if (control.channel !== undefined) {
-                for (seed in source_by_seed) {
-                    source = source_by_seed[seed];
+                for (const seed in source_by_seed) {
+                    const source = source_by_seed[seed];
                     if (!source.window.closed)
                         source.window.postMessage(message, origin);
                 }
             } else if (control.command == "hint") {
-                if (control.credential) {
-                    if (index.privileges)
-                        index.privileges.update(control);
-                }
+                /* This is where we handle hint messages directed at
+                 * the shell.  Right now, there aren't any.
+                 */
             }
 
         /* Forward message to relevant frame */
         } else if (channel) {
-            pos = channel.indexOf('!');
+            const pos = channel.indexOf('!');
             if (pos !== -1) {
-                seed = channel.substring(0, pos + 1);
-                source = source_by_seed[seed];
+                const seed = channel.substring(0, pos + 1);
+                const source = source_by_seed[seed];
                 if (source) {
                     if (!source.window.closed)
                         source.window.postMessage(message, origin);
@@ -236,12 +275,12 @@ function Router(index) {
     }, false);
 
     function perform_jump(child, control) {
-        var current_frame = index.current_frame();
+        const current_frame = index.current_frame();
         if (child !== window) {
             if (!current_frame || current_frame.contentWindow != child)
                 return;
         }
-        var str = control.location || "";
+        let str = control.location || "";
         if (str[0] != "/")
             str = "/" + str;
         if (control.host)
@@ -250,22 +289,26 @@ function Router(index) {
     }
 
     function perform_track(child) {
-        var hash;
-        var current_frame = index.current_frame();
+        const current_frame = index.current_frame();
         /* Note that we ignore tracknig for old shell code */
         if (current_frame && current_frame.contentWindow === child &&
             child.name && child.name.indexOf("/shell/shell") === -1) {
-            hash = child.location.hash;
+            let hash = child.location.hash;
             if (hash.indexOf("#") === 0)
                 hash = hash.substring(1);
             if (hash === "/")
                 hash = "";
-            index.jump({ hash: hash });
+            /* The browser has already pushed an appropriate entry to
+               the history, so let's just replace it with our custom
+               state object.
+            */
+            const state = Object.assign({}, index.retrieve_state(), { hash });
+            index.navigate(state, true);
         }
     }
 
     function on_unload(ev) {
-        var source;
+        let source;
         if (ev.target.defaultView)
             source = source_by_name[ev.target.defaultView.name];
         else if (ev.view)
@@ -275,21 +318,21 @@ function Router(index) {
     }
 
     function on_hashchange(ev) {
-        var source = source_by_name[ev.target.name];
+        const source = source_by_name[ev.target.name];
         if (source)
             perform_track(source.window);
     }
 
     function on_load(ev) {
-        var source = source_by_name[ev.target.contentWindow.name];
+        const source = source_by_name[ev.target.contentWindow.name];
         if (source)
             perform_track(source.window);
     }
 
     function unregister(source) {
-        var child = source.window;
+        const child = source.window;
         cockpit.kill(null, child.name);
-        var frame = child.frameElement;
+        const frame = child.frameElement;
         if (frame)
             frame.removeEventListener("load", on_load);
         /* This is often invalid when the window is closed */
@@ -302,10 +345,10 @@ function Router(index) {
     }
 
     function register(child) {
-        var host, page;
-        var name = child.name || "";
+        let host, page;
+        const name = child.name || "";
         if (name.indexOf("cockpit1:") === 0) {
-            var parts = name.substring(9).split("/");
+            const parts = name.substring(9).split("/");
             host = parts[0];
             page = parts.slice(1).join("/");
         }
@@ -315,19 +358,19 @@ function Router(index) {
         }
 
         unique_id += 1;
-        var seed = (cockpit.transport.options["channel-seed"] || "undefined:") + unique_id + "!";
-        var source = {
-            name: name,
+        const seed = (cockpit.transport.options["channel-seed"] || "undefined:") + unique_id + "!";
+        const source = {
+            name,
             window: child,
             channel_seed: seed,
             default_host: host,
-            page: page,
+            page,
             inited: false,
         };
         source_by_seed[seed] = source;
         source_by_name[name] = source;
 
-        var frame = child.frameElement;
+        const frame = child.frameElement;
         frame.addEventListener("load", on_load);
         child.addEventListener("unload", on_unload);
         child.addEventListener("hashchange", on_hashchange);
@@ -349,9 +392,9 @@ function Router(index) {
         if (event.origin !== origin)
             return;
 
-        var forward_command = false;
-        var data = event.data;
-        var child = event.source;
+        let forward_command = false;
+        let data = event.data;
+        const child = event.source;
         if (!child)
             return;
 
@@ -367,7 +410,7 @@ function Router(index) {
         if (typeof data !== "string")
             return;
 
-        var source, control;
+        let source, control;
 
         /*
          * On Internet Explorer we see Access Denied when non Cockpit
@@ -401,9 +444,12 @@ function Router(index) {
                     source = register(child);
                 }
                 if (source) {
-                    var reply = $.extend({ }, cockpit.transport.options,
-                                         { command: "init", host: source.default_host, "channel-seed": source.channel_seed }
-                    );
+                    const reply = {
+                        ...cockpit.transport.options,
+                        command: "init",
+                        host: source.default_host,
+                        "channel-seed": source.channel_seed,
+                    };
                     child.postMessage("\n" + JSON.stringify(reply), origin);
                     source.inited = true;
 
@@ -428,7 +474,8 @@ function Router(index) {
                 index.show_oops();
                 return;
             } else if (control.command == "notify") {
-                index.handle_notifications(source.default_host, source.page, control);
+                if (source)
+                    index.handle_notifications(source.default_host, source.page, control);
                 return;
 
             /* Only control messages with a channel are forwardable */
@@ -453,17 +500,16 @@ function Router(index) {
 
     self.start = function start(messages) {
         window.addEventListener("message", message_handler, false);
-        for (var i = 0, len = messages.length; i < len; i++)
+        for (let i = 0, len = messages.length; i < len; i++)
             message_handler(messages[i]);
     };
 
     self.hint = function hint(child, data) {
-        var message;
-        var source = source_by_name[child.name];
+        const source = source_by_name[child.name];
         /* This is often invalid when the window is closed */
         if (source && source.inited && !source.window.closed) {
             data.command = "hint";
-            message = "\n" + JSON.stringify(data);
+            const message = "\n" + JSON.stringify(data);
             source.window.postMessage(message, origin);
         }
     };
@@ -475,45 +521,113 @@ function Router(index) {
  * prototype. That function will be called by Frames and
  * Router to actually perform any navigation action.
  *
- * As a convenience, common menu items can be setup by adding the
- * selector to be used to hook them up. The accepted selectors
- * are.
- * oops_sel, logout_sel, language_sel, brand_sel, about_sel,
- * user_sel, account_sel
- *
  * Emits "disconnect" and "expect_restart" signals, that should be
  * handled by the caller.
  */
 function Index() {
-    var self = this;
-    var current_frame;
+    const self = this;
+    let current_frame;
+
+    cockpit.event_target(self);
 
     if (typeof self.navigate !== "function")
         throw Error("Index requires a prototype with a navigate function");
 
-    self.frames = new Frames(self);
+    /* Session timing out after inactivity */
+    let session_final_timer = null;
+    let session_timeout = 0;
+    let current_idle_time = 0;
+    let final_countdown = 30000; // last 30 seconds
+    let title = "";
+    const standard_login = window.localStorage['standard-login'];
+
+    self.has_oops = false;
+
+    function sessionTimeout() {
+        current_idle_time += 5000;
+        if (!session_final_timer && current_idle_time >= session_timeout - final_countdown) {
+            title = document.title;
+            sessionFinalTimeout();
+        }
+    }
+
+    let session_timeout_dialog_root = null;
+
+    function updateFinalCountdown() {
+        const remaining_secs = Math.floor(final_countdown / 1000);
+        const timeout_text = cockpit.format(_("You will be logged out in $0 seconds."), remaining_secs);
+        document.title = "(" + remaining_secs + ") " + title;
+        if (!session_timeout_dialog_root)
+            session_timeout_dialog_root = createRoot(document.getElementById('session-timeout-dialog'));
+        session_timeout_dialog_root.render(React.createElement(TimeoutModal, {
+            onClose: () => {
+                window.clearTimeout(session_final_timer);
+                session_final_timer = null;
+                document.title = title;
+                resetTimer();
+                session_timeout_dialog_root.unmount();
+                session_timeout_dialog_root = null;
+                final_countdown = 30000;
+            },
+            text: timeout_text,
+        }));
+    }
+
+    function sessionFinalTimeout() {
+        final_countdown -= 1000;
+        if (final_countdown > 0) {
+            updateFinalCountdown();
+            session_final_timer = window.setTimeout(sessionFinalTimeout, 1000);
+        } else {
+            cockpit.logout(true, _("You have been logged out due to inactivity."));
+        }
+    }
+
+    /* Auto-logout idle timer */
+    function resetTimer(ev) {
+        if (!session_final_timer) {
+            current_idle_time = 0;
+        }
+    }
+
+    function setupIdleResetTimers(win) {
+        win.addEventListener("mousemove", resetTimer, false);
+        win.addEventListener("mousedown", resetTimer, false);
+        win.addEventListener("keypress", resetTimer, false);
+        win.addEventListener("touchmove", resetTimer, false);
+        win.addEventListener("scroll", resetTimer, false);
+    }
+
+    cockpit.dbus(null, { bus: "internal" }).call("/config", "cockpit.Config", "GetUInt", ["Session", "IdleTimeout", 0, 240, 0], [])
+            .then(result => {
+                session_timeout = result[0] * 60000;
+                if (session_timeout > 0 && standard_login) {
+                    setupIdleResetTimers(window);
+                    window.setInterval(sessionTimeout, 5000);
+                }
+            })
+            .catch(e => {
+                if (e.message.indexOf("GetUInt not available") === -1)
+                    console.warn(e.message);
+            });
+
+    self.frames = new Frames(self, setupIdleResetTimers);
     self.router = new Router(self);
 
     /* Watchdog for disconnect */
-    var watchdog = cockpit.channel({ payload: "null" });
-    $(watchdog).on("close", function(event, options) {
-        var watchdog_problem = options.problem || "disconnected";
+    const watchdog = cockpit.channel({ payload: "null" });
+    watchdog.addEventListener("close", (event, options) => {
+        const watchdog_problem = options.problem || "disconnected";
         console.warn("transport closed: " + watchdog_problem);
-        $(self).triggerHandler("disconnect", watchdog_problem);
+        self.dispatchEvent("disconnect", watchdog_problem);
     });
 
-    /* Handles an href link as seen below */
-    $(document).on("click", "a[href]", function(ev) {
-        var a = this;
-        if (!a.host || window.location.host === a.host) {
-            self.jump(a.getAttribute('href'));
-            ev.preventDefault();
-        }
-    });
-
-    var old_onerror = window.onerror;
+    const old_onerror = window.onerror;
     window.onerror = function cockpit_error_handler(msg, url, line) {
-        self.show_oops();
+        // Errors with url == "" are not logged apparently, so let's
+        // not show the "Oops" for them either.
+        if (url != "")
+            self.show_oops();
         if (old_onerror)
             return old_onerror(msg, url, line);
         return false;
@@ -544,12 +658,12 @@ function Index() {
      * generating a string for a link.
      */
     function encode(state, sidebar, with_root) {
-        var path = [];
+        const path = [];
         if (state.host && (sidebar || state.host !== "localhost"))
             path.push("@" + state.host);
         if (state.component)
             path.push.apply(path, state.component.split("/"));
-        var string = cockpit.location.encode(path, null, with_root);
+        let string = cockpit.location.encode(path, null, with_root);
         if (state.hash && state.hash !== "/")
             string += "#" + state.hash;
         return string;
@@ -557,15 +671,15 @@ function Index() {
 
     /* Decodes navigate state from a string */
     function decode(string) {
-        var state = { version: "v1", hash: "" };
-        var pos = string.indexOf("#");
+        const state = { version: "v1", hash: "" };
+        const pos = string.indexOf("#");
         if (pos !== -1) {
             state.hash = string.substring(pos + 1);
             string = string.substring(0, pos);
         }
         if (string[0] != '/')
             string = "/" + string;
-        var path = cockpit.location.decode(string);
+        const path = cockpit.location.decode(string);
         if (path[0] && path[0][0] == "@") {
             state.host = path.shift().substring(1);
             state.sidebar = true;
@@ -578,48 +692,8 @@ function Index() {
         return state;
     }
 
-    function build_navbar() {
-        var navbar = $("#main-navbar");
-        navbar.on("click", function () {
-            navbar.parent().toggleClass("clicked", true);
-        });
-
-        navbar.on("mouseout", function () {
-            navbar.parent().toggleClass("clicked", false);
-        });
-
-        function links(component) {
-            var sm = $("<span class='fa'>")
-                    .attr("data-toggle", "tooltip")
-                    .attr("role", "presentation")
-                    .attr("title", "")
-                    .attr("data-original-title", component.label);
-
-            if (component.icon)
-                sm.addClass(component.icon);
-            else
-                sm.addClass("first-letter").text(component.label);
-
-            var value = $("<span class='list-group-item-value'>")
-                    .text(component.label);
-
-            var a = $("<a>")
-                    .attr("href", self.href({ host: "localhost", component: component.path }))
-                    .append(sm)
-                    .append(value);
-
-            return $("<li class='dashboard-link list-group-item'>")
-                    .attr("data-component", component.path)
-                    .append(a);
-        }
-
-        var local_compiled = new CompiledComponents();
-        local_compiled.load(cockpit.manifests, "dashboard");
-        navbar.append(local_compiled.ordered("dashboard").map(links));
-    }
-
     self.retrieve_state = function() {
-        var state = window.history.state;
+        let state = window.history.state;
         if (!state || state.version !== "v1") {
             if (shell_embedded)
                 state = decode("/" + window.location.hash);
@@ -630,17 +704,14 @@ function Index() {
     };
 
     function lookup_component_hash(address, component) {
-        var iframe, src;
-
         if (!address)
             address = "localhost";
 
-        var list = self.frames.iframes[address];
-        if (list)
-            iframe = list[component];
+        const list = self.frames.iframes[address];
+        const iframe = list ? list[component] : undefined;
 
         if (iframe) {
-            src = iframe.getAttribute('src');
+            const src = iframe.getAttribute('src');
             if (src)
                 return src.split("#")[1];
         }
@@ -667,26 +738,32 @@ function Index() {
         if (typeof (state) === "string")
             state = decode(state);
 
-        var current = self.retrieve_state();
+        const current = self.retrieve_state();
 
         /* Make sure we have the data we need */
         if (!state.host)
             state.host = current.host || "localhost";
+
+        // When switching hosts, check if we left from some page
+        if (!state.component && state.host !== current.host) {
+            const host_frames = self.frames.iframes[state.host] || {};
+            const active = Object.keys(host_frames)
+                    .filter(k => host_frames[k].getAttribute('data-active') === 'true');
+            if (active.length > 0)
+                state.component = active[0];
+        }
+
         if (!("component" in state))
             state.component = current.component || "";
 
-        var target;
-        var history = window.history;
-        var frame_change = (state.host !== current.host ||
+        const history = window.history;
+        const frame_change = (state.host !== current.host ||
                             state.component !== current.component);
 
         if (frame_change && !state.hash)
             state.hash = lookup_component_hash(state.host, state.component);
 
-        if (shell_embedded)
-            target = window.location;
-        else
-            target = encode(state, null, true);
+        const target = shell_embedded ? window.location : encode(state, null, true);
 
         if (replace) {
             history.replaceState(state, "", target);
@@ -695,6 +772,7 @@ function Index() {
 
         if (frame_change || state.hash !== current.hash) {
             history.pushState(state, "", target);
+            document.getElementById("nav-system").classList.remove("interact");
             self.navigate(state, true);
             return true;
         }
@@ -708,8 +786,8 @@ function Index() {
     };
 
     self.show_oops = function () {
-        if (self.oops_sel)
-            $(self.oops_sel).show();
+        self.has_oops = true;
+        self.dispatchEvent("update");
     };
 
     self.current_frame = function (frame) {
@@ -726,224 +804,62 @@ function Index() {
     };
 
     self.start = function() {
-        /* window.messages is initialized in shell/indexes.js */
-        var messages = window.messages;
+        /* window.messages is initialized in shell/indexes.jsx */
+        const messages = window.messages;
         if (messages)
             messages.cancel();
         self.router.start(messages || []);
     };
 
     self.ready = function () {
-        $(window).on("popstate", function(ev) {
+        window.addEventListener("popstate", ev => {
             self.navigate(ev.state, true);
         });
 
-        build_navbar();
-        self.navigate();
+        self.navigate(null, true);
         cockpit.translate();
-        $("body").show();
+        document.body.removeAttribute("hidden");
     };
 
     self.expect_restart = function (host) {
-        $(self).triggerHandler("expect_restart", host);
+        self.dispatchEvent("expect_restart", host);
     };
-
-    /* Menu items */
-    /* The oops bar */
-    function setup_oops(id) {
-        var oops = $(id);
-        if (!oops)
-            return;
-        oops.children("a").on("click", function() {
-            $("#error-popup-title").text(_("Unexpected error"));
-            var details = _("Cockpit had an unexpected internal error. <br/><br/>You can try restarting Cockpit by pressing refresh in your browser. The javascript console contains details about this error (<b>Ctrl-Shift-J</b> in most browsers).");
-            $("#error-popup-message").html(details);
-            $('#error-popup').modal('show');
-        });
-    }
-
-    function css_content(elt) {
-        var styles, i;
-        var style, content;
-        var el_style, el_content;
-
-        if (elt)
-            style = window.getComputedStyle(elt, ":before");
-        if (!style)
-            return;
-
-        content = style.content;
-        // Old branding had style on the element itself
-        if (!content) {
-            el_style = window.getComputedStyle(elt);
-            if (el_style)
-                el_content = el_style.content;
-        }
-
-        // getComputedStyle is broken for pseudo elements
-        // in Phantomjs and some old browsers
-        // those support the depricated getMatchedCSSRules
-        if (!content && style.length === 0 && window.getMatchedCSSRules) {
-            styles = window.getMatchedCSSRules(elt, ":before") || [];
-            for (i = 0; i < styles.length; i++) {
-                if (styles[i].style.content) {
-                    content = styles[i].style.content;
-                    break;
-                }
-            }
-        }
-
-        return content || el_content;
-    }
-
-    /* Branding */
-    function setup_brand(id, default_title) {
-        var elt = $(id)[0];
-        var os_release = {};
-        try {
-            os_release = JSON.parse(window.localStorage['os-release'] || "{}");
-        } catch (ex) {
-            console.warn("Couldn't parse os-release", ex);
-        }
-
-        var len;
-        var content = css_content(elt);
-        if (content && content != "none" && content != "normal") {
-            len = content.length;
-            if ((content[0] === '"' || content[0] === '\'') &&
-                len > 2 && content[len - 1] === content[0])
-                content = content.substr(1, len - 2);
-            elt.innerHTML = cockpit.format(content, os_release) || default_title;
-            return $(elt).text();
-        } else {
-            elt.removeAttribute("class");
-        }
-    }
-
-    /* Logout link */
-    function setup_logout(id) {
-        $(id).on("click", function() {
-            cockpit.logout();
-        });
-    }
-
-    /* Display language dialog */
-    function setup_language(id) {
-        /*
-         * Note that we don't go ahead and load all the po files in order
-         * to produce this list. Perhaps we would include it somewhere in a
-         * separate automatically generated file. Need to see.
-         */
-        var manifest = cockpit.manifests.shell || { };
-        $(".display-language-menu").toggle(!!manifest.locales);
-        var language = document.cookie.replace(/(?:(?:^|.*;\s*)CockpitLang\s*=\s*([^;]*).*$)|^.*$/, "$1");
-        if (!language)
-            language = "en-us";
-
-        $('html').attr('lang', language);
-
-        $.each(manifest.locales || { }, function(code, name) {
-            var el = $("<option>").text(name)
-                    .val(code);
-            if (code == language)
-                el.attr("selected", "true");
-            $("#display-language-list").append(el);
-        });
-
-        $("#display-language-select-button").on("click", function(event) {
-            var code_to_select = $("#display-language-list").val();
-            var cookie = "CockpitLang=" + encodeURIComponent(code_to_select) +
-                         "; path=/; expires=Sun, 16 Jul 3567 06:23:41 GMT";
-            document.cookie = cookie;
-            window.localStorage.setItem("cockpit.lang", code_to_select);
-            window.location.reload(true);
-            return false;
-        });
-
-        $(id).on("shown.bs.modal", function() {
-            $("display-language-list").focus();
-        });
-    }
-
-    /* About dialog */
-    function setup_about(id) {
-        $(cockpit.info).on("changed", function() {
-            $(id).text(cockpit.info.version);
-        });
-    }
-
-    /* Account link */
-    function setup_account(id, user) {
-        $(id).on("click", function() {
-            self.jump({ host: "localhost", component: "users", hash: "/" + user.name });
-        })
-                .show();
-    }
-
-    function setup_killer(id) {
-        $(id).on("click", function(ev) {
-            if (ev && ev.button === 0)
-                showDialog(self.frames);
-        });
-    }
-
-    /* User information */
-    function setup_user(id, user) {
-        $(id).text(user.full_name || user.name || '???');
-    }
-
-    if (self.oops_sel)
-        setup_oops(self.oops_sel);
-
-    if (self.logout_sel)
-        setup_logout(self.logout_sel);
-
-    if (self.language_sel)
-        setup_language(self.language_sel);
-
-    var cal_title;
-    if (self.brand_sel) {
-        cal_title = setup_brand(self.brand_sel, self.default_title);
-        if (cal_title && !self.skip_brand_title)
-            self.default_title = cal_title;
-    }
-
-    if (self.about_sel)
-        setup_about(self.about_sel);
-    if (self.killer_sel)
-        setup_killer(self.killer_sel);
-
-    if (self.user_sel || self.account_sel) {
-        cockpit.user().done(function (user) {
-            if (self.user_sel)
-                setup_user(self.user_sel, user);
-            if (self.account_sel)
-                setup_account(self.account_sel, user);
-        });
-    }
 }
 
 function CompiledComponents() {
-    var self = this;
+    const self = this;
     self.items = {};
 
     self.load = function(manifests, section) {
-        $.each(manifests || { }, function(name, manifest) {
-            $.each(manifest[section] || { }, function(prop, info) {
-                var item = {
-                    section: section,
+        Object.entries(manifests || { }).forEach(([name, manifest]) => {
+            Object.entries(manifest[section] || { }).forEach(([prop, info]) => {
+                const item = {
+                    section,
                     label: cockpit.gettext(info.label) || prop,
                     order: info.order === undefined ? 1000 : info.order,
-                    icon: info.icon,
-                    wants: info.wants
+                    docs: info.docs,
+                    keywords: info.keywords || [{ matches: [] }],
+                    keyword: { score: -1 }
                 };
+
+                // Always first keyword should be page name
+                const page_name = item.label.toLowerCase();
+                if (item.keywords[0].matches.indexOf(page_name) < 0)
+                    item.keywords[0].matches.unshift(page_name);
+
+                // Keywords from manifest have different defaults than are usual
+                item.keywords.forEach(i => {
+                    i.weight = i.weight || 3;
+                    i.translate = i.translate === undefined ? true : i.translate;
+                });
+
                 if (info.path)
                     item.path = info.path.replace(/\.html$/, "");
                 else
                     item.path = name + "/" + prop;
 
                 /* Split out any hash in the path */
-                var pos = item.path.indexOf("#");
+                const pos = item.path.indexOf("#");
                 if (pos !== -1) {
                     item.hash = item.path.substr(pos + 1);
                     item.path = item.path.substr(0, pos);
@@ -960,14 +876,13 @@ function CompiledComponents() {
     };
 
     self.ordered = function(section) {
-        var x;
-        var list = [];
-        for (x in self.items) {
+        const list = [];
+        for (const x in self.items) {
             if (!section || self.items[x].section === section)
                 list.push(self.items[x]);
         }
         list.sort(function(a, b) {
-            var ret = a.order - b.order;
+            let ret = a.order - b.order;
             if (ret === 0)
                 ret = a.label.localeCompare(b.label);
             return ret;
@@ -976,8 +891,7 @@ function CompiledComponents() {
     };
 
     self.search = function(prop, value) {
-        var x;
-        for (x in self.items) {
+        for (const x in self.items) {
             if (self.items[x][prop] === value)
                 return self.items[x];
         }
@@ -994,7 +908,7 @@ function follow(arg) {
     }
 }
 
-var zz_value;
+let zz_value;
 
 /* For debugging utility in the index window */
 Object.defineProperties(window, {
@@ -1006,7 +920,7 @@ Object.defineProperties(window, {
 });
 
 export function new_index_from_proto(proto) {
-    var o = new Object(proto); // eslint-disable-line no-new-object
+    const o = new Object(proto); // eslint-disable-line no-new-object
     Index.call(o);
     return o;
 }
