@@ -19,10 +19,14 @@
 
 import cockpit from "cockpit";
 import React from "react";
-import moment from "moment";
-import { Alert } from "@patternfly/react-core";
-
-import { dialog_open, TeardownMessage, TextInput, ComboBox, CheckBoxes } from "./dialog.jsx";
+import { Alert } from "@patternfly/react-core/dist/esm/components/Alert/index.js";
+import { Card, CardActions, CardBody, CardHeader, CardTitle } from "@patternfly/react-core/dist/esm/components/Card/index.js";
+import { Text, TextVariants } from "@patternfly/react-core/dist/esm/components/Text/index.js";
+import { DescriptionList, DescriptionListDescription, DescriptionListGroup, DescriptionListTerm } from "@patternfly/react-core/dist/esm/components/DescriptionList/index.js";
+import {
+    dialog_open, TextInput, ComboBox, CheckBoxes,
+    StopProcessesMessage, stop_processes_danger_message
+} from "./dialog.jsx";
 import * as format from "./format-dialog.jsx";
 
 import { StdDetailsLayout } from "./details.jsx";
@@ -38,27 +42,12 @@ function nfs_busy_dialog(client, dialog_title, entry, error, action_title, actio
                 Body: error.toString()
             });
         } else {
-            const sessions = [];
-            const services = [];
-            users.forEach((u) => {
-                var since = moment.duration(-u.since * 1000).humanize(true);
-                if (u.unit.endsWith(".scope")) {
-                    sessions.push({ Name: u.desc, Command: u.cmd.substr(0, 200), Since: since });
-                } else {
-                    services.push({ Name: u.desc, Unit: u.unit, Since: since });
-                }
-            });
-
             dialog_open({
                 Title: dialog_title,
-                Body: TeardownMessage({
-                    Teardown: {
-                        Sessions: sessions,
-                        Services: services
-                    }
-                }),
+                Teardown: <StopProcessesMessage users={users} />,
                 Action: {
                     DangerButton: true,
+                    Danger: stop_processes_danger_message(users),
                     Title: action_title,
                     action: function () {
                         return action(users);
@@ -69,10 +58,10 @@ function nfs_busy_dialog(client, dialog_title, entry, error, action_title, actio
     }
 
     client.nfs.entry_users(entry)
-            .done(function (users) {
+            .then(function (users) {
                 show(users);
             })
-            .fail(function () {
+            .catch(function () {
                 show([]);
             });
 }
@@ -80,9 +69,9 @@ function nfs_busy_dialog(client, dialog_title, entry, error, action_title, actio
 function get_exported_directories(server) {
     return cockpit.spawn(["showmount", "-e", "--no-headers", server], { err: "message" })
             .then(function (output) {
-                var dirs = [];
+                const dirs = [];
                 output.split("\n").forEach(function (line) {
-                    var d = line.split(" ")[0];
+                    const d = line.split(" ")[0];
                     if (d)
                         dirs.push(d);
                 });
@@ -91,14 +80,14 @@ function get_exported_directories(server) {
 }
 
 export function nfs_fstab_dialog(client, entry) {
-    var mount_options = entry ? entry.fields[3] : "defaults";
-    var split_options = format.parse_options(mount_options == "defaults" ? "" : mount_options);
-    var opt_auto = !format.extract_option(split_options, "noauto");
-    var opt_ro = format.extract_option(split_options, "ro");
-    var extra_options = format.unparse_options(split_options);
+    const mount_options = entry ? entry.fields[3] : "defaults";
+    const split_options = format.parse_options(mount_options == "defaults" ? "" : mount_options);
+    const opt_auto = !format.extract_option(split_options, "noauto");
+    const opt_ro = format.extract_option(split_options, "ro");
+    const extra_options = format.unparse_options(split_options);
 
     function mounting_options(vals) {
-        var opts = [];
+        let opts = [];
         if (!vals.mount_options.auto)
             opts.push("noauto");
         if (vals.mount_options.ro)
@@ -111,7 +100,10 @@ export function nfs_fstab_dialog(client, entry) {
     function show(busy) {
         let alert = null;
         if (busy)
-            alert = <Alert isInline variant="danger" title={_("This NFS mount is in use and only its options can be changed.")} />;
+            alert = <>
+                <Alert isInline variant="danger" title={_("This NFS mount is in use and only its options can be changed.")} />
+                <br />
+            </>;
 
         let server_to_check = null;
         let server_check_timeout = null;
@@ -125,16 +117,16 @@ export function nfs_fstab_dialog(client, entry) {
                 dlg.set_options("remote", { choices: [] });
                 get_exported_directories(server).then(choices => {
                     if (server == server_to_check)
-                        dlg.set_options("remote", { choices: choices });
+                        dlg.set_options("remote", { choices });
                 });
             }, delay);
         }
 
         const dlg = dialog_open({
-            Title: entry ? _("NFS Mount") : _("New NFS Mount"),
+            Title: entry ? _("NFS mount") : _("New NFS mount"),
             Body: alert,
             Fields: [
-                TextInput("server", _("Server Address"),
+                TextInput("server", _("Server address"),
                           {
                               value: entry ? entry.fields[0].split(":")[0] : "",
                               validate: function (val) {
@@ -143,7 +135,7 @@ export function nfs_fstab_dialog(client, entry) {
                               },
                               disabled: busy
                           }),
-                ComboBox("remote", _("Path on Server"),
+                ComboBox("remote", _("Path on server"),
                          {
                              value: entry ? entry.fields[0].split(":")[1] : "",
                              validate: function (val) {
@@ -155,7 +147,7 @@ export function nfs_fstab_dialog(client, entry) {
                              disabled: busy,
                              choices: [],
                          }),
-                TextInput("dir", _("Local Mount Point"),
+                TextInput("dir", _("Local mount point"),
                           {
                               value: entry ? entry.fields[1] : "",
                               validate: function (val) {
@@ -166,7 +158,7 @@ export function nfs_fstab_dialog(client, entry) {
                               },
                               disabled: busy
                           }),
-                CheckBoxes("mount_options", _("Mount Options"),
+                CheckBoxes("mount_options", _("Mount options"),
                            {
                                fields: [
                                    { title: _("Mount at boot"), tag: "auto" },
@@ -186,18 +178,18 @@ export function nfs_fstab_dialog(client, entry) {
                     check_server(dlg, vals.server, 500);
             },
             Action: {
-                Title: entry ? _("Apply") : _("Add"),
+                Title: entry ? _("Save") : _("Add"),
                 action: function (vals) {
-                    var location = cockpit.location;
-                    var fields = [vals.server + ":" + vals.remote,
+                    const location = cockpit.location;
+                    const fields = [vals.server + ":" + vals.remote,
                         vals.dir,
                         entry ? entry.fields[2] : "nfs",
                         mounting_options(vals) || "defaults"];
                     if (entry) {
                         return client.nfs.update_entry(entry, fields)
-                                .done(function () {
+                                .then(function () {
                                     if (entry.fields[0] != fields[0] ||
-                                                                 entry.fields[1] != fields[1])
+                                    entry.fields[1] != fields[1])
                                         location.go(["nfs", fields[0], fields[1]]);
                                 });
                     } else
@@ -212,10 +204,10 @@ export function nfs_fstab_dialog(client, entry) {
 
     if (entry) {
         client.nfs.entry_users(entry)
-                .done(function (users) {
+                .then(function (users) {
                     show(users.length > 0);
                 })
-                .fail(function () {
+                .catch(function () {
                     show(false);
                 });
     } else
@@ -224,14 +216,14 @@ export function nfs_fstab_dialog(client, entry) {
 
 export class NFSDetails extends React.Component {
     render() {
-        var client = this.props.client;
-        var entry = this.props.entry;
-        var fsys_size;
+        const client = this.props.client;
+        const entry = this.props.entry;
+        let fsys_size;
         if (entry.mounted)
             fsys_size = client.nfs.get_fsys_size(entry);
 
         function checked(error_title, promise) {
-            promise.fail(function (error) {
+            promise.catch(error => {
                 dialog_open({
                     Title: error_title,
                     Body: error.toString()
@@ -245,20 +237,20 @@ export class NFSDetails extends React.Component {
         }
 
         function unmount() {
-            var location = cockpit.location;
+            const location = cockpit.location;
             client.nfs.unmount_entry(entry)
-                    .done(function () {
+                    .then(function () {
                         if (!entry.fstab)
                             location.go("/");
                     })
-                    .fail(function (error) {
+                    .catch(function (error) {
                         nfs_busy_dialog(client,
                                         _("Unable to unmount filesystem"),
                                         entry, error,
-                                        _("Stop and Unmount"),
+                                        _("Stop and unmount"),
                                         function (users) {
                                             return client.nfs.stop_and_unmount_entry(users, entry)
-                                                    .done(function () {
+                                                    .then(function () {
                                                         if (!entry.fstab)
                                                             location.go("/");
                                                     });
@@ -271,60 +263,73 @@ export class NFSDetails extends React.Component {
         }
 
         function remove() {
-            var location = cockpit.location;
+            const location = cockpit.location;
             client.nfs.remove_entry(entry)
-                    .done(function () {
+                    .then(function () {
                         location.go("/");
                     })
-                    .fail(function (error) {
+                    .catch(function (error) {
                         nfs_busy_dialog(client,
                                         _("Unable to remove mount"),
                                         entry, error,
                                         _("Stop and remove"),
                                         function (users) {
                                             return client.nfs.stop_and_remove_entry(users, entry)
-                                                    .done(function () {
+                                                    .then(function () {
                                                         location.go("/");
                                                     });
                                         });
                     });
         }
 
-        var header = (
-            <div className="panel panel-default">
-                <div className="panel-heading">
-                    {entry.fields[0]}
-                    <span className="pull-right">
-                        { entry.mounted
-                            ? <StorageButton onClick={unmount}>{_("Unmount")}</StorageButton>
-                            : <StorageButton onClick={mount}>{_("Mount")}</StorageButton>
-                        }
-                        { "\n" }
-                        { entry.fstab
-                            ? [
-                                <StorageButton key="1" onClick={edit}>{_("Edit")}</StorageButton>,
-                                "\n",
-                                <StorageButton key="2" onClick={remove} kind="danger">{_("Remove")}</StorageButton>
-                            ] : null
-                        }
-                    </span>
-                </div>
-                <div className="panel-body">
-                    <div className="ct-form">
-                        <label className="control-label">{_("Server")}</label>
-                        <div>{entry.fields[0]}</div>
+        const header = (
+            <Card>
+                <CardHeader>
+                    <CardTitle>
+                        <Text component={TextVariants.h2}>{entry.fields[0]}</Text>
+                    </CardTitle>
+                    <CardActions>
+                        <>
+                            { entry.mounted
+                                ? <StorageButton onClick={unmount}>{_("Unmount")}</StorageButton>
+                                : <StorageButton onClick={mount}>{_("Mount")}</StorageButton>
+                            }
+                            { "\n" }
+                            { entry.fstab
+                                ? [
+                                    <StorageButton key="1" onClick={edit}>{_("Edit")}</StorageButton>,
+                                    "\n",
+                                    <StorageButton key="2" onClick={remove} kind="danger">{_("Remove")}</StorageButton>
+                                ]
+                                : null
+                            }
+                        </>
+                    </CardActions>
+                </CardHeader>
+                <CardBody>
+                    <DescriptionList className="pf-m-horizontal-on-sm">
+                        <DescriptionListGroup>
+                            <DescriptionListTerm className="control-DescriptionListTerm">{_("Server")}</DescriptionListTerm>
+                            <DescriptionListDescription>{entry.fields[0]}</DescriptionListDescription>
+                        </DescriptionListGroup>
 
-                        <label className="control-label">{_("Mount Point")}</label>
-                        <div>{entry.fields[1]}</div>
+                        <DescriptionListGroup>
+                            <DescriptionListTerm className="control-DescriptionListTerm">{_("Mount point")}</DescriptionListTerm>
+                            <DescriptionListDescription>{entry.fields[1]}</DescriptionListDescription>
+                        </DescriptionListGroup>
 
-                        <label className="control-label">{_("Size")}</label>
-                        { entry.mounted
-                            ? <StorageUsageBar stats={fsys_size} critical={0.95} />
-                            : _("--")
-                        }
-                    </div>
-                </div>
-            </div>
+                        <DescriptionListGroup>
+                            <DescriptionListTerm className="control-DescriptionListTerm">{_("Size")}</DescriptionListTerm>
+                            <DescriptionListDescription className="pf-u-align-self-center">
+                                { entry.mounted
+                                    ? <StorageUsageBar stats={fsys_size} critical={0.95} block={entry.fields[1]} />
+                                    : "--"
+                                }
+                            </DescriptionListDescription>
+                        </DescriptionListGroup>
+                    </DescriptionList>
+                </CardBody>
+            </Card>
         );
 
         return <StdDetailsLayout client={client} header={header} />;
