@@ -20,9 +20,7 @@
 import cockpit from "cockpit";
 
 import * as service from "service";
-import moment from "moment";
-
-moment.locale(cockpit.language);
+import * as timeformat from "timeformat";
 
 const _ = cockpit.gettext;
 const C_ = cockpit.gettext;
@@ -35,10 +33,10 @@ export function compare_versions(a, b) {
         return str.split(".").map(function (s) { return s ? parseInt(s, 10) : 0 });
     }
 
-    var a_ints = to_ints(a);
-    var b_ints = to_ints(b);
-    var len = Math.min(a_ints.length, b_ints.length);
-    var i;
+    const a_ints = to_ints(a);
+    const b_ints = to_ints(b);
+    const len = Math.min(a_ints.length, b_ints.length);
+    let i;
 
     for (i = 0; i < len; i++) {
         if (a_ints[i] == b_ints[i])
@@ -49,10 +47,10 @@ export function compare_versions(a, b) {
     return a_ints.length - b_ints.length;
 }
 
-export var hostnamed = cockpit.dbus("org.freedesktop.hostname1").proxy();
+export let hostnamed = cockpit.dbus("org.freedesktop.hostname1").proxy();
 
 // for unit tests
-var orig_hostnamed;
+let orig_hostnamed;
 
 export function mock_hostnamed(value) {
     if (value) {
@@ -64,7 +62,7 @@ export function mock_hostnamed(value) {
 }
 
 export function array_find(array, pred) {
-    for (var i = 0; i < array.length; i++)
+    for (let i = 0; i < array.length; i++)
         if (pred(array[i]))
             return array[i];
     return undefined;
@@ -87,43 +85,46 @@ export function encode_filename(decoded) {
 }
 
 export function fmt_size(bytes) {
-    return cockpit.format_bytes(bytes, 1024);
+    return cockpit.format_bytes(bytes);
 }
 
 export function fmt_size_long(bytes) {
-    var with_binary_unit = cockpit.format_bytes(bytes, 1024);
-    var with_decimal_unit = cockpit.format_bytes(bytes, 1000);
+    const with_decimal_unit = cockpit.format_bytes(bytes, 1000);
+    const with_binary_unit = cockpit.format_bytes(bytes, 1024);
     /* Translators: Used in "..." */
-    return with_binary_unit + ", " + with_decimal_unit + ", " + bytes + " " + C_("format-bytes", "bytes");
+    return with_decimal_unit + ", " + with_binary_unit + ", " + bytes + " " + C_("format-bytes", "bytes");
 }
 
 export function fmt_rate(bytes_per_sec) {
-    return cockpit.format_bytes_per_sec(bytes_per_sec, 1024);
+    return cockpit.format_bytes_per_sec(bytes_per_sec);
 }
 
 export function format_temperature(kelvin) {
-    var celcius = kelvin - 273.15;
-    var fahrenheit = 9.0 * celcius / 5.0 + 32.0;
-    return celcius.toFixed(1) + "° C / " + fahrenheit.toFixed(1) + "° F";
+    const celsius = kelvin - 273.15;
+    const fahrenheit = 9.0 * celsius / 5.0 + 32.0;
+    return celsius.toFixed(1) + "° C / " + fahrenheit.toFixed(1) + "° F";
 }
 
 export function format_fsys_usage(used, total) {
-    var text = "";
-    var units = 1024;
-    var parts = cockpit.format_bytes(total, units, true);
+    let text = "";
+    let parts = cockpit.format_bytes(total, undefined, { separate: true, precision: 2 });
     text = " / " + parts.join(" ");
-    units = parts[1];
+    const unit = parts[1];
 
-    parts = cockpit.format_bytes(used, units, true);
+    parts = cockpit.format_bytes(used, unit, { separate: true, precision: 2 });
     return parts[0] + text;
 }
 
 export function format_delay(d) {
-    return moment.duration(d).humanize();
+    return timeformat.distanceToNow(new Date().valueOf() + d);
 }
 
 export function format_size_and_text(size, text) {
-    return cockpit.format(_("${size} ${desc}"), { size: fmt_size(size), desc: text });
+    return fmt_size(size) + " " + text;
+}
+
+export function validate_mdraid_name(name) {
+    return validate_lvm2_name(name);
 }
 
 export function validate_lvm2_name(name) {
@@ -131,7 +132,7 @@ export function validate_lvm2_name(name) {
         return _("Name cannot be empty.");
     if (name.length > 127)
         return _("Name cannot be longer than 127 characters.");
-    var m = name.match(/[^a-zA-Z0-9+._-]/);
+    const m = name.match(/[^a-zA-Z0-9+._-]/);
     if (m) {
         if (m[0].search(/\s+/) === -1)
             return cockpit.format(_("Name cannot contain the character '$0'."), m[0]);
@@ -141,15 +142,15 @@ export function validate_lvm2_name(name) {
 }
 
 export function validate_fsys_label(label, type) {
-    var fs_label_max = {
-        xfs:   12,
-        ext4:  16,
-        vfat:  11,
+    const fs_label_max = {
+        xfs: 12,
+        ext4: 16,
+        vfat: 11,
         ntfs: 128,
     };
 
-    var limit = fs_label_max[type.replace("luks+", "")];
-    var bytes = cockpit.utf8_encoder().encode(label);
+    const limit = fs_label_max[type.replace("luks+", "")];
+    const bytes = cockpit.utf8_encoder().encode(label);
     if (limit && bytes.length > limit) {
         // Let's not confuse people with encoding issues unless
         // they use funny characters.
@@ -168,15 +169,21 @@ export function mdraid_name(mdraid) {
     if (!mdraid.Name)
         return "";
 
-    var parts = mdraid.Name.split(":");
+    const parts = mdraid.Name.split(":");
 
     if (parts.length != 2)
         return mdraid.Name;
 
-    /* if we call hostnamed too early, before the dbus.proxy() promise is fulfilled,
-     * it will not be valid yet; it's too inconvenient to make this
-     * function asynchronous, so just don't show the host name in this case */
-    if (hostnamed.StaticHostname === undefined || parts[0] == hostnamed.StaticHostname)
+    /* Check the static (from /etc/hostname) and transient (acquired from DHCP server via
+     * NetworkManager → hostnamed, may not exist) host name -- if either one matches, we
+     * consider the RAID a local one and just show the device name.
+     * Otherwise it's a remote one, and include the host in the name.
+     *
+     * However: if we call hostnamed too early, before the dbus.proxy() promise is
+     * fulfilled, it will not be valid yet (hostnamed properties are undefined);
+     * it's too inconvenient to make this function asynchronous, so just don't
+     * show the host name in this case. */
+    if (hostnamed.StaticHostname === undefined || parts[0] == hostnamed.StaticHostname || parts[0] == hostnamed.Hostname)
         return parts[1];
     else
         return cockpit.format(_("$name (from $host)"),
@@ -187,26 +194,26 @@ export function mdraid_name(mdraid) {
 }
 
 export function lvol_name(lvol) {
-    var type;
+    let type;
     if (lvol.Type == "pool")
-        type = _("Pool for Thin Logical Volumes");
+        type = _("Pool for thin logical volumes");
     else if (lvol.ThinPool != "/")
-        type = _("Thin Logical Volume");
+        type = _("Thin logical volume");
     else if (lvol.Origin != "/")
-        type = _("Logical Volume (Snapshot)");
+        type = _("Logical volume (snapshot)");
     else
-        type = _("Logical Volume");
+        type = _("Logical volume");
     return cockpit.format('$0 "$1"', type, lvol.Name);
 }
 
 export function drive_name(drive) {
-    var name_parts = [];
+    const name_parts = [];
     if (drive.Vendor)
         name_parts.push(drive.Vendor);
     if (drive.Model)
         name_parts.push(drive.Model);
 
-    var name = name_parts.join(" ");
+    let name = name_parts.join(" ");
     if (drive.Serial)
         name += " (" + drive.Serial + ")";
     else if (drive.WWN)
@@ -216,7 +223,7 @@ export function drive_name(drive) {
 }
 
 export function get_block_link_parts(client, path) {
-    var is_part, is_crypt, is_lvol;
+    let is_part, is_crypt, is_lvol;
 
     while (true) {
         if (client.blocks_part[path] && client.blocks_ptable[client.blocks_part[path].Table]) {
@@ -232,25 +239,25 @@ export function get_block_link_parts(client, path) {
     if (client.blocks_lvm2[path] && client.lvols[client.blocks_lvm2[path].LogicalVolume])
         is_lvol = true;
 
-    var block = client.blocks[path];
+    const block = client.blocks[path];
     if (!block)
         return;
 
-    var location, link;
+    let location, link;
     if (client.mdraids[block.MDRaid]) {
         location = ["mdraid", client.mdraids[block.MDRaid].UUID];
-        link = cockpit.format(_("RAID Device $0"), mdraid_name(client.mdraids[block.MDRaid]));
+        link = cockpit.format(_("RAID device $0"), mdraid_name(client.mdraids[block.MDRaid]));
     } else if (client.blocks_lvm2[path] &&
                client.lvols[client.blocks_lvm2[path].LogicalVolume] &&
                client.vgroups[client.lvols[client.blocks_lvm2[path].LogicalVolume].VolumeGroup]) {
-        var target = client.vgroups[client.lvols[client.blocks_lvm2[path].LogicalVolume].VolumeGroup].Name;
+        const target = client.vgroups[client.lvols[client.blocks_lvm2[path].LogicalVolume].VolumeGroup].Name;
         location = ["vg", target];
-        link = cockpit.format(_("Volume Group $0"), target);
+        link = cockpit.format(_("LVM2 volume group $0"), target);
     } else {
-        var vdo = client.vdo_overlay.find_by_block(block);
+        const vdo = client.legacy_vdo_overlay.find_by_block(block);
         if (vdo) {
             location = ["vdo", vdo.name];
-            link = cockpit.format(_("VDO Device $0"), vdo.name);
+            link = cockpit.format(_("VDO device $0"), vdo.name);
         } else {
             location = [block_name(block).replace(/^\/dev\//, "")];
             if (client.drives[block.Drive])
@@ -261,13 +268,13 @@ export function get_block_link_parts(client, path) {
     }
 
     // Partitions of logical volumes are shown as just logical volumes.
-    var format;
+    let format;
     if (is_lvol && is_crypt)
-        format = _("Encrypted Logical Volume of $0");
+        format = _("Encrypted logical volume of $0");
     else if (is_part && is_crypt)
-        format = _("Encrypted Partition of $0");
+        format = _("Encrypted partition of $0");
     else if (is_lvol)
-        format = _("Logical Volume of $0");
+        format = _("Logical volume of $0");
     else if (is_part)
         format = _("Partition of $0");
     else if (is_crypt)
@@ -276,27 +283,27 @@ export function get_block_link_parts(client, path) {
         format = "$0";
 
     return {
-        location: location,
-        format: format,
-        link: link
+        location,
+        format,
+        link
     };
 }
 
 export function go_to_block(client, path) {
-    var parts = get_block_link_parts(client, path);
+    const parts = get_block_link_parts(client, path);
     cockpit.location.go(parts.location);
 }
 
 export function get_partitions(client, block) {
-    var partitions = client.blocks_partitions[block.path];
+    const partitions = client.blocks_partitions[block.path];
 
     function process_level(level, container_start, container_size) {
-        var n;
-        var last_end = container_start;
-        var total_end = container_start + container_size;
-        var block, start, size, is_container, is_contained;
+        let n;
+        let last_end = container_start;
+        const total_end = container_start + container_size;
+        let block, start, size, is_container, is_contained;
 
-        var result = [];
+        const result = [];
 
         function append_free_space(start, size) {
             // There is a lot of rounding and aligning going on in
@@ -319,7 +326,7 @@ export function get_partitions(client, block) {
             // MiB gap.)
 
             if (size >= 3 * 1024 * 1024) {
-                result.push({ type: 'free', start: start, size: size });
+                result.push({ type: 'free', start, size });
             }
         }
 
@@ -345,11 +352,13 @@ export function get_partitions(client, block) {
             append_free_space(last_end, start - last_end);
             if (is_container) {
                 result.push({
-                    type: 'container', block: block, size: size,
+                    type: 'container',
+                    block,
+                    size,
                     partitions: process_level(level + 1, start, size)
                 });
             } else {
-                result.push({ type: 'block', block: block });
+                result.push({ type: 'block', block });
             }
             last_end = start + size;
         }
@@ -364,10 +373,10 @@ export function get_partitions(client, block) {
 
 export function get_available_spaces(client) {
     function is_free(path) {
-        var block = client.blocks[path];
-        var block_ptable = client.blocks_ptable[path];
-        var block_part = client.blocks_part[path];
-        var block_pvol = client.blocks_pvol[path];
+        const block = client.blocks[path];
+        const block_ptable = client.blocks_ptable[path];
+        const block_part = client.blocks_part[path];
+        const block_pvol = client.blocks_pvol[path];
 
         function has_fs_label() {
             if (!block.IdUsage)
@@ -386,8 +395,8 @@ export function get_available_spaces(client) {
                 // Broken multipath drive
                 return true;
             }
-            var members = client.drives_multipath_blocks[block.Drive];
-            for (var i = 0; i < members.length; i++) {
+            const members = client.drives_multipath_blocks[block.Drive];
+            for (let i = 0; i < members.length; i++) {
                 if (members[i] == block)
                     return true;
             }
@@ -395,7 +404,11 @@ export function get_available_spaces(client) {
         }
 
         function is_vdo_backing_dev() {
-            return !!client.vdo_overlay.find_by_backing_block(block);
+            return !!client.legacy_vdo_overlay.find_by_backing_block(block);
+        }
+
+        function is_swap() {
+            return !!block && client.blocks_swap[path];
         }
 
         return (!block.HintIgnore &&
@@ -403,38 +416,42 @@ export function get_available_spaces(client) {
                 !has_fs_label() &&
                 !is_mpath_member() &&
                 !is_vdo_backing_dev() &&
+                !is_swap() &&
                 !block_ptable &&
                 !(block_part && block_part.IsContainer));
     }
 
     function make(path) {
-        var block = client.blocks[path];
-        var parts = get_block_link_parts(client, path);
-        var text = cockpit.format(parts.format, parts.link);
-        return { type: 'block', block: block, size: block.Size, desc: text };
+        const block = client.blocks[path];
+        const parts = get_block_link_parts(client, path);
+        const text = cockpit.format(parts.format, parts.link);
+        return { type: 'block', block, size: block.Size, desc: text };
     }
 
-    var spaces = Object.keys(client.blocks).filter(is_free)
+    const spaces = Object.keys(client.blocks).filter(is_free)
             .sort(make_block_path_cmp(client))
             .map(make);
 
     function add_free_spaces(block) {
-        var parts = get_partitions(client, block);
-        var i, p, link_parts, text;
+        const parts = get_partitions(client, block);
+        let i, p, link_parts, text;
         for (i in parts) {
             p = parts[i];
             if (p.type == 'free') {
                 link_parts = get_block_link_parts(client, block.path);
                 text = cockpit.format(link_parts.format, link_parts.link);
                 spaces.push({
-                    type: 'free', block: block, start: p.start, size: p.size,
+                    type: 'free',
+                    block,
+                    start: p.start,
+                    size: p.size,
                     desc: cockpit.format(_("unpartitioned space on $0"), text)
                 });
             }
         }
     }
 
-    for (var p in client.blocks_ptable)
+    for (const p in client.blocks_ptable)
         add_free_spaces(client.blocks[p]);
 
     return spaces;
@@ -445,15 +462,36 @@ export function prepare_available_spaces(client, spcs) {
         if (spc.type == 'block')
             return cockpit.resolve(spc.block.path);
         else if (spc.type == 'free') {
-            var block_ptable = client.blocks_ptable[spc.block.path];
+            const block_ptable = client.blocks_ptable[spc.block.path];
             return block_ptable.CreatePartition(spc.start, spc.size, "", "", { });
         }
     }
 
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    return cockpit.all(spcs.map(prepare));
+    return Promise.all(spcs.map(prepare));
+}
+
+export function is_snap(client, block) {
+    const block_fsys = client.blocks_fsys[block.path];
+    return block_fsys && block_fsys.MountPoints.map(decode_filename).some(mp => mp.indexOf("/snap/") == 0 || mp.indexOf("/var/lib/snapd/snap/") == 0);
+}
+
+export function get_other_devices(client) {
+    return Object.keys(client.blocks).filter(path => {
+        const block = client.blocks[path];
+        const block_part = client.blocks_part[path];
+        const block_lvm2 = client.blocks_lvm2[path];
+
+        return ((!block_part || block_part.Table == "/") &&
+                block.Drive == "/" &&
+                block.CryptoBackingDevice == "/" &&
+                block.MDRaid == "/" &&
+                (!block_lvm2 || block_lvm2.LogicalVolume == "/") &&
+                !block.HintIgnore &&
+                block.Size > 0 &&
+                !client.legacy_vdo_overlay.find_by_block(block) &&
+                !client.blocks_stratis_fsys[block.path] &&
+                !is_snap(client, block));
+    });
 }
 
 /* Comparison function for sorting lists of block devices.
@@ -478,7 +516,7 @@ export function make_block_path_cmp(client) {
     };
 }
 
-var multipathd_service;
+let multipathd_service;
 
 export function get_multipathd_service () {
     if (!multipathd_service)
@@ -499,10 +537,12 @@ export function get_parent(client, path) {
         return client.blocks_lvm2[path].LogicalVolume;
     if (client.lvols[path] && client.vgroups[client.lvols[path].VolumeGroup])
         return client.lvols[path].VolumeGroup;
+    if (client.blocks_stratis_fsys[path])
+        return client.blocks_stratis_fsys[path].Pool;
 }
 
 export function get_direct_parent_blocks(client, path) {
-    var parent = get_parent(client, path);
+    let parent = get_parent(client, path);
     if (!parent)
         return [];
     if (client.blocks[parent])
@@ -513,20 +553,22 @@ export function get_direct_parent_blocks(client, path) {
         parent = client.lvols[parent].VolumeGroup;
     if (client.vgroups[parent])
         return client.vgroups_pvols[parent].map(function (pv) { return pv.path });
+    if (client.stratis_pools[parent])
+        return client.stratis_pool_blockdevs[parent].map(bd => client.slashdevs_block[bd.Devnode].path);
     return [];
 }
 
 export function get_parent_blocks(client, path) {
-    var direct_parents = get_direct_parent_blocks(client, path);
-    var direct_and_indirect_parents = flatten(direct_parents.map(function (p) {
+    const direct_parents = get_direct_parent_blocks(client, path);
+    const direct_and_indirect_parents = flatten(direct_parents.map(function (p) {
         return get_parent_blocks(client, p);
     }));
     return [path].concat(direct_and_indirect_parents);
 }
 
 export function is_netdev(client, path) {
-    var block = client.blocks[path];
-    var drive = block && client.drives[block.Drive];
+    const block = client.blocks[path];
+    const drive = block && client.drives[block.Drive];
     if (drive && drive.Vendor == "LIO-ORG")
         return true;
     if (block && block.Major == 43) // NBD
@@ -535,7 +577,7 @@ export function is_netdev(client, path) {
 }
 
 function get_children(client, path) {
-    var children = [];
+    const children = [];
 
     if (client.blocks_cleartext[path]) {
         children.push(client.blocks_cleartext[path].path);
@@ -549,7 +591,7 @@ function get_children(client, path) {
     }
 
     if (client.blocks_part[path] && client.blocks_part[path].IsContainer) {
-        var ptable_path = client.blocks_part[path].Table;
+        const ptable_path = client.blocks_part[path].Table;
         client.blocks_partitions[ptable_path].forEach(function (part) {
             if (part.IsContained)
                 children.push(part.path);
@@ -563,127 +605,123 @@ function get_children(client, path) {
         });
     }
 
+    if (client.lvols_pool_members[path]) {
+        for (const lvol of client.lvols_pool_members[path]) {
+            const block = client.lvols_block[lvol.path];
+            if (block)
+                children.push(block.path);
+        }
+    }
+
+    if (client.stratis_pools[path]) {
+        client.stratis_pool_filesystems[path].forEach(function (fsys) {
+            const block = client.slashdevs_block[fsys.Devnode];
+            if (block)
+                children.push(block.path);
+        });
+    }
+
     return children;
 }
 
-export function get_active_usage(client, path) {
-    function get_usage(path) {
-        var block = client.blocks[path];
-        var fsys = client.blocks_fsys[path];
-        var mdraid = block && client.mdraids[block.MDRaidMember];
-        var pvol = client.blocks_pvol[path];
-        var vgroup = pvol && client.vgroups[pvol.VolumeGroup];
-        var vdo = block && client.vdo_overlay.find_by_backing_block(block);
+export function get_active_usage(client, path, top_action, child_action) {
+    function get_usage(path, level) {
+        const block = client.blocks[path];
+        const fsys = client.blocks_fsys[path];
+        const mdraid = block && client.mdraids[block.MDRaidMember];
+        const pvol = client.blocks_pvol[path];
+        const vgroup = pvol && client.vgroups[pvol.VolumeGroup];
+        const vdo = block && client.legacy_vdo_overlay.find_by_backing_block(block);
+        const stratis_blockdev = block && client.blocks_stratis_blockdev[path];
+        const stratis_pool = stratis_blockdev && client.stratis_pools[stratis_blockdev.Pool];
 
-        var usage = flatten(get_children(client, path).map(get_usage));
+        const usage = flatten(get_children(client, path).map(p => get_usage(p, level + 1)));
 
-        if (fsys && fsys.MountPoints.length > 0)
-            usage.push({
-                usage: 'mounted',
-                block: block,
-                fsys: fsys
+        function get_actions(teardown_action) {
+            const actions = [];
+            if (teardown_action)
+                actions.push(teardown_action);
+            const global_action = (level == 0 || (block && client.blocks[block.CryptoBackingDevice] && level == 1)) ? top_action : child_action || top_action;
+            if (global_action)
+                actions.push(global_action);
+            return actions;
+        }
+
+        if (fsys && fsys.MountPoints.length > 0) {
+            fsys.MountPoints.forEach(mp => {
+                usage.push({
+                    level,
+                    usage: 'mounted',
+                    block,
+                    location: decode_filename(mp),
+                    actions: get_actions(_("unmount")),
+                    blocking: false,
+                });
+            });
+        } else if (mdraid) {
+            const active_state = array_find(mdraid.ActiveDevices, function (as) {
+                return as[0] == block.path;
             });
-
-        if (mdraid)
             usage.push({
+                level,
                 usage: 'mdraid-member',
-                block: block,
-                mdraid: mdraid
+                block,
+                mdraid,
+                location: mdraid_name(mdraid.Name),
+                actions: get_actions(_("remove from RAID")),
+                blocking: !(active_state && active_state[1] < 0)
             });
-
-        if (vgroup)
+        } else if (vgroup) {
             usage.push({
+                level,
                 usage: 'pvol',
-                block: block,
-                pvol: pvol,
-                vgroup: vgroup
+                block,
+                vgroup,
+                pvol,
+                location: vgroup.Name,
+                actions: get_actions(_("remove from LVM2")),
+                blocking: pvol.FreeSize != pvol.Size
             });
-
-        if (vdo)
+        } else if (vdo) {
             usage.push({
+                level,
                 usage: 'vdo-backing',
-                block: block,
-                vdo: vdo
+                block,
+                vdo,
+                location: vdo.name,
+                blocking: true
             });
+        } else if (stratis_pool) {
+            usage.push({
+                level,
+                usage: 'stratis-pool-member',
+                block,
+                stratis_pool,
+                location: stratis_pool.Name,
+                blocking: true
+            });
+        } else if (block && !client.blocks_cleartext[block.path]) {
+            usage.push({
+                level,
+                usage: 'none',
+                block,
+                actions: get_actions(null),
+                blocking: false
+            });
+        }
 
         return usage;
     }
 
-    // Prepare the result for the dialogs
+    let usage = get_usage(path, 0);
 
-    var usage = get_usage(path);
+    if (usage.length == 1 && usage[0].level == 0 && usage[0].usage == "none")
+        usage = [];
 
-    var res = {
-        raw: usage,
-        Teardown: {
-            Mounts: [],
-            MDRaidMembers: [],
-            PhysicalVolumes: []
-        },
-        Blocking: {
-            Mounts: [],
-            MDRaidMembers: [],
-            PhysicalVolumes: [],
-            VDOs: []
-        }
-    };
+    usage.Blocking = usage.some(u => u.blocking);
+    usage.Teardown = usage.some(u => !u.blocking);
 
-    usage.forEach(function (use) {
-        var entry, active_state;
-
-        if (use.usage == 'mounted') {
-            res.Teardown.Mounts.push({
-                Name: block_name(use.block),
-                MountPoint: decode_filename(use.fsys.MountPoints[0])
-            });
-        } else if (use.usage == 'mdraid-member') {
-            entry = {
-                Name: block_name(use.block),
-                MDRaid: mdraid_name(use.mdraid)
-            };
-            active_state = array_find(use.mdraid.ActiveDevices, function (as) {
-                return as[0] == use.block.path;
-            });
-            if (active_state && active_state[1] < 0)
-                res.Teardown.MDRaidMembers.push(entry);
-            else
-                res.Blocking.MDRaidMembers.push(entry);
-        } else if (use.usage == 'pvol') {
-            entry = {
-                Name: block_name(use.block),
-                VGroup: use.vgroup.Name
-            };
-            if (use.pvol.FreeSize == use.pvol.Size) {
-                res.Teardown.PhysicalVolumes.push(entry);
-            } else {
-                res.Blocking.PhysicalVolumes.push(entry);
-            }
-        } else if (use.usage == 'vdo-backing') {
-            entry = {
-                Name: block_name(use.block),
-                VDO: use.vdo.name
-            };
-            res.Blocking.VDOs.push(entry);
-        }
-    });
-
-    res.Teardown.HasMounts = res.Teardown.Mounts.length > 0;
-    res.Teardown.HasMDRaidMembers = res.Teardown.MDRaidMembers.length > 0;
-    res.Teardown.HasPhysicalVolumes = res.Teardown.PhysicalVolumes.length > 0;
-
-    if (!res.Teardown.HasMounts && !res.Teardown.HasMDRaidMembers && !res.Teardown.HasPhysicalVolumes)
-        res.Teardown = null;
-
-    res.Blocking.HasMounts = res.Blocking.Mounts.length > 0;
-    res.Blocking.HasMDRaidMembers = res.Blocking.MDRaidMembers.length > 0;
-    res.Blocking.HasPhysicalVolumes = res.Blocking.PhysicalVolumes.length > 0;
-    res.Blocking.HasVDOs = res.Blocking.VDOs.length > 0;
-
-    if (!res.Blocking.HasMounts && !res.Blocking.HasMDRaidMembers && !res.Blocking.HasPhysicalVolumes &&
-        !res.Blocking.HasVDOs)
-        res.Blocking = null;
-
-    return res;
+    return usage;
 }
 
 export function teardown_active_usage(client, usage) {
@@ -697,29 +735,17 @@ export function teardown_active_usage(client, usage) {
     // condition upfront by reshuffling the data structures.
 
     function unmount(mounteds) {
-        // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-        // https://github.com/cockpit-project/cockpit/issues/10956
-        // eslint-disable-next-line cockpit/no-cockpit-all
-        return cockpit.all(mounteds.map(function (m) {
-            if (m.fsys.MountPoints.length > 0)
-                return m.fsys.Unmount({});
-            else
-                return cockpit.resolve();
+        return Promise.all(mounteds.map(m => {
+            return client.unmount_at(m.location, m.users);
         }));
     }
 
     function mdraid_remove(members) {
-        // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-        // https://github.com/cockpit-project/cockpit/issues/10956
-        // eslint-disable-next-line cockpit/no-cockpit-all
-        return cockpit.all(members.map(function (m) {
-            return m.mdraid.RemoveDevice(m.block.path, { wipe: { t: 'b', v: true } });
-        }));
+        return Promise.all(members.map(m => m.mdraid.RemoveDevice(m.block.path, { wipe: { t: 'b', v: true } })));
     }
 
     function pvol_remove(pvols) {
-        var by_vgroup = { };
-        var p;
+        const by_vgroup = { };
         pvols.forEach(function (p) {
             if (!by_vgroup[p.vgroup.path])
                 by_vgroup[p.vgroup.path] = [];
@@ -727,49 +753,47 @@ export function teardown_active_usage(client, usage) {
         });
 
         function handle_vg(p) {
-            var vg = client.vgroups[p];
-            var pvs = by_vgroup[p];
+            const vg = client.vgroups[p];
+            const pvs = by_vgroup[p];
             // If we would remove all physical volumes of a volume
             // group, remove the whole volume group instead.
             if (pvs.length == client.vgroups_pvols[p].length) {
-                return vg.Delete({ 'tear-down': { t: 'b', v: true } });
+                return vg.Delete(true, { 'tear-down': { t: 'b', v: true } }).then(reload_systemd);
             } else {
-                // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-                // https://github.com/cockpit-project/cockpit/issues/10956
-                // eslint-disable-next-line cockpit/no-cockpit-all
-                return cockpit.all(pvs.map(function (pv) {
-                    return vg.RemoveDevice(pv.path, true, {});
-                }));
+                return Promise.all(pvs.map(pv => vg.RemoveDevice(pv.path, true, {})));
             }
         }
 
-        for (p in by_vgroup)
-            handle_vg(p);
+        return Promise.all(Object.keys(by_vgroup).map(handle_vg));
     }
 
-    // We can't use Promise.all() here until cockpit is able to dispatch es2015 promises
-    // https://github.com/cockpit-project/cockpit/issues/10956
-    // eslint-disable-next-line cockpit/no-cockpit-all
-    return cockpit.all([unmount(usage.raw.filter(function(use) { return use.usage == "mounted" })),
-        mdraid_remove(usage.raw.filter(function(use) { return use.usage == "mdraid-member" })),
-        pvol_remove(usage.raw.filter(function(use) { return use.usage == "pvol" }))
-    ]);
-}
-
-export function get_config(name, def) {
-    if (cockpit.manifests.storage && cockpit.manifests.storage.config) {
-        var val = cockpit.manifests.storage.config[name];
-        return val !== undefined ? val : def;
-    } else {
-        return def;
-    }
+    return Promise.all(Array.prototype.concat(
+        unmount(usage.filter(function(use) { return use.usage == "mounted" })),
+        mdraid_remove(usage.filter(function(use) { return use.usage == "mdraid-member" })),
+        pvol_remove(usage.filter(function(use) { return use.usage == "pvol" }))
+    ));
 }
 
 // TODO - generalize this to arbitrary number of arguments (when needed)
 export function fmt_to_array(fmt, arg) {
-    var index = fmt.indexOf("$0");
+    const index = fmt.indexOf("$0");
     if (index >= 0)
         return [fmt.slice(0, index), arg, fmt.slice(index + 2)];
     else
         return [fmt];
 }
+
+export function reload_systemd() {
+    return cockpit.spawn(["systemctl", "daemon-reload"], { superuser: "require", err: "message" });
+}
+
+export function is_mounted_synch(block) {
+    return (cockpit.spawn(["findmnt", "-n", "-o", "TARGET", "-S", decode_filename(block.Device)],
+                          { superuser: true, err: "message" })
+            .then(data => data.trim())
+            .catch(() => false));
+}
+
+export function for_each_async(arr, func) {
+    return arr.reduce((promise, elt) => promise.then(() => func(elt)), Promise.resolve());
+}
