@@ -24,9 +24,11 @@
 
 #include "cockpitinternalmetrics.h"
 
-#include "common/cockpittest.h"
+#include "testlib/cockpittest.h"
 #include "common/cockpitjson.h"
-#include "common/mock-transport.h"
+#include "testlib/mock-transport.h"
+
+#include <unistd.h>
 
 typedef struct {
   MockTransport *transport;
@@ -93,32 +95,32 @@ setup (TestCase *tc,
 }
 
 static GBytes *
-recv_bytes (TestCase *tc)
+recv_bytes (MockTransport *transport)
 {
   GBytes *msg;
-  while ((msg = mock_transport_pop_channel (tc->transport, "1234")) == NULL)
+  while ((msg = mock_transport_pop_channel (transport, "1234")) == NULL)
     g_main_context_iteration (NULL, TRUE);
   return msg;
 }
 
 static JsonObject *
-recv_object (TestCase *tc)
+recv_object (MockTransport *transport)
 {
-  GBytes *msg = recv_bytes (tc);
+  GBytes *msg = recv_bytes (transport);
   JsonObject *res = cockpit_json_parse_bytes (msg, NULL);
   g_assert (res != NULL);
   return res;
 }
 
 static JsonArray *
-recv_array (TestCase *tc)
+recv_array (MockTransport *transport)
 {
   GBytes *msg;
   GError *error = NULL;
   JsonArray *array;
   JsonNode *node;
 
-  msg = recv_bytes (tc);
+  msg = recv_bytes (transport);
   node = cockpit_json_parse (g_bytes_get_data (msg, NULL), g_bytes_get_size (msg), &error);
   g_assert_no_error (error);
   g_assert_cmpint (json_node_get_node_type (node), ==, JSON_NODE_ARRAY);
@@ -154,7 +156,7 @@ assert_sample_msg (const char *domain,
                    TestCase *tc,
                    const gchar *json_str)
 {
-  JsonArray *array = recv_array (tc);
+  JsonArray *array = recv_array (tc->transport);
   _cockpit_assert_json_eq_msg (domain, file, line, func, array, json_str);
   json_array_unref (array);
 }
@@ -223,7 +225,7 @@ test_compression (TestCase *tc,
                                "  'interval': 1000"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc,    0, 2, 0.0, 0.0);
   assert_sample (tc, "[[0,0]]");
@@ -253,7 +255,7 @@ test_compression_reset (TestCase *tc,
                                "  'interval': 1000"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc,    0, 2, 0.0, 0.0);
   assert_sample (tc, "[[0,0]]");
@@ -261,7 +263,7 @@ test_compression_reset (TestCase *tc,
   assert_sample (tc, "[[]]");
 
   cockpit_metrics_send_meta (tc->channel, meta, TRUE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc, 2000, 2, 0.0, 0.0);
   assert_sample (tc, "[[0,0]]");
@@ -282,7 +284,7 @@ test_derive_delta (TestCase *tc,
                                "  'interval': 100"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc,    0, 1, 0.0);
   assert_sample (tc, "[[false]]");
@@ -302,7 +304,7 @@ test_derive_delta (TestCase *tc,
   assert_sample (tc, "[[0]]");
 
   cockpit_metrics_send_meta (tc->channel, meta, TRUE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc,  800, 1, 30.0);
   assert_sample (tc, "[[false]]");
@@ -331,7 +333,7 @@ test_derive_rate_no_interpolate (TestCase *tc,
                                "  'interval': 100"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc,    0, 1, 0.0);
   assert_sample (tc, "[[false]]");
@@ -351,7 +353,7 @@ test_derive_rate_no_interpolate (TestCase *tc,
   assert_sample (tc, "[[0]]");
 
   cockpit_metrics_send_meta (tc->channel, meta, TRUE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_sample (tc,  800, 1, 30.0);
   assert_sample (tc, "[[false]]");
@@ -389,7 +391,7 @@ assert_2_approx_samples_msg (const char *domain,
                              double val1,
                              double val2)
 {
-  JsonArray *array = recv_array (tc);
+  JsonArray *array = recv_array (tc->transport);
   JsonArray *sub_array;
 
   if (json_array_get_length (array) != 1)
@@ -441,7 +443,7 @@ test_interpolate (TestCase *tc,
                                "  'interval': 100"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   // rising by 10 for every 100 ms, with non-equally spaced samples
 
@@ -472,7 +474,7 @@ test_instances (TestCase *tc,
                                "  'interval': 1000"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_instance_sample (tc,    0, 2, 0.0, 0.0);
   assert_sample (tc, "[[[0,0]]]");
@@ -502,7 +504,7 @@ test_dynamic_instances (TestCase *tc,
                                "  'interval': 100"
                                "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   send_instance_sample (tc,    0, 1, 0.0);
   assert_sample (tc, "[[[false]]]");
@@ -520,7 +522,7 @@ test_dynamic_instances (TestCase *tc,
                    "  'interval': 100"
                    "}");
   cockpit_metrics_send_meta (tc->channel, meta, FALSE);
-  json_object_unref (recv_object (tc));
+  json_object_unref (recv_object (tc->transport));
 
   /* Instance 'a' is now at a different index.  The 'delta' derivation
      should continue to work, but no compression should happen.
@@ -719,6 +721,187 @@ test_deprecated_net_all (void)
   g_object_unref (transport);
 }
 
+static void
+test_cgroup (void)
+{
+  MockTransport *transport = mock_transport_new ();
+  CockpitChannel *channel;
+  JsonObject *options = json_obj ("{ 'metrics': [ { 'name': 'cgroup.memory.usage' }, "
+                                  "               { 'name': 'cgroup.memory.limit' }, "
+                                  "               { 'name': 'cgroup.memory.sw-usage' }, "
+                                  "               { 'name': 'cgroup.memory.sw-limit' }, "
+                                  "               { 'name': 'cgroup.cpu.usage' }, "
+                                  "               { 'name': 'cgroup.cpu.shares' } ], "
+                                  "  'interval': 1000"
+                                  "}");
+  GBytes *msg;
+  JsonObject *res, *description;
+  JsonArray *metrics, *instances;
+  JsonArray *samples;
+  GError *error = NULL;
+
+  g_signal_connect (transport, "closed", G_CALLBACK (on_transport_closed), NULL);
+
+  channel = g_object_new (cockpit_internal_metrics_get_type (),
+                          "transport", transport,
+                          "id", "1234",
+                          "options", options,
+                          NULL);
+
+  cockpit_channel_prepare (channel);
+
+  /* receive meta information */
+  while ((msg = mock_transport_pop_channel (transport, "1234")) == NULL)
+    g_main_context_iteration (NULL, TRUE);
+  res = cockpit_json_parse_bytes (msg, &error);
+  g_assert_no_error (error);
+  g_assert (res != NULL);
+  g_assert (json_object_has_member (res, "metrics"));
+
+  /* metrics should have the form [{"name":"...","instances":["name1", ...]}] */
+  metrics = json_object_get_array_member (res, "metrics");
+  g_assert_cmpint (json_array_get_length (metrics), ==, 6);
+  description = json_array_get_object_element (metrics, 0);
+  g_assert (description);
+  g_assert_cmpstr (json_object_get_string_member (description, "name"), ==, "cgroup.memory.usage");
+  g_assert_cmpstr (json_object_get_string_member (description, "units"), ==, "bytes");
+
+  /* we can't assert any contents about instances and samples, as build environments may not even have
+   * any cgroup controller; just make sure that the data structure is as expected */
+  instances = json_object_get_array_member (description, "instances");
+  g_assert_cmpint (json_array_get_length (instances), >=, 0);
+
+  /* next message should have some actual values; looks like [[...],[...],...]]] */
+  samples = recv_array (transport);
+  g_assert_cmpint (json_array_get_length (samples), ==, 1);
+  g_assert_cmpint (json_array_get_length (json_array_get_array_element (samples, 0)), ==, 6);
+
+  json_array_unref (samples);
+
+  json_object_unref (res);
+  g_object_unref (channel);
+  json_object_unref (options);
+  g_object_unref (transport);
+}
+
+static void
+test_cpu_cores (void)
+{
+  MockTransport *transport = mock_transport_new ();
+  CockpitChannel *channel;
+
+  JsonObject *options = json_obj ("{ 'metrics': [ { 'name': 'cpu.core.nice', 'derive': 'rate' }, "
+                                  "               { 'name': 'cpu.core.user', 'derive': 'rate' }, "
+                                  "               { 'name': 'cpu.core.system', 'derive': 'rate' }, "
+                                  "               { 'name': 'cpu.core.iowait', 'derive': 'rate' } ], "
+                                  "  'interval': 1000"
+                                  "}");
+  GBytes *msg;
+  JsonObject *res, *description;
+  JsonArray *metrics, *instances, *all, *nice;
+  JsonArray *samples;
+  GError *error = NULL;
+
+  g_signal_connect (transport, "closed", G_CALLBACK (on_transport_closed), NULL);
+
+  channel = g_object_new (cockpit_internal_metrics_get_type (),
+                          "transport", transport,
+                          "id", "1234",
+                          "options", options,
+                          NULL);
+
+  cockpit_channel_prepare (channel);
+
+  /* receive meta information */
+  while ((msg = mock_transport_pop_channel (transport, "1234")) == NULL)
+    g_main_context_iteration (NULL, TRUE);
+  res = cockpit_json_parse_bytes (msg, &error);
+  g_assert_no_error (error);
+  g_assert (res != NULL);
+  g_assert (json_object_has_member (res, "metrics"));
+
+  /* metrics should have the form [{"name":"...","instances":[[1, 0, ...], ...]}] */
+  metrics = json_object_get_array_member (res, "metrics");
+  g_assert_cmpint (json_array_get_length (metrics), ==, 4);
+  description = json_array_get_object_element (metrics, 0);
+  g_assert (description);
+  g_assert_cmpstr (json_object_get_string_member (description, "name"), ==, "cpu.core.nice");
+  g_assert_cmpstr (json_object_get_string_member (description, "units"), ==, "millisec");
+
+  /* Array contains value for each core */
+  instances = json_object_get_array_member (description, "instances");
+  g_assert_cmpint (json_array_get_length (instances), ==, sysconf(_SC_NPROCESSORS_ONLN));
+
+  /* Value of each core is somewhere between 0 and 1000 */
+  samples = recv_array (transport);
+  g_assert_cmpint (json_array_get_length (samples), ==, 1);
+  all = json_array_get_array_element (samples, 0);
+  g_assert_cmpint (json_array_get_length (all), ==, 4);
+  nice = json_array_get_array_element (all, 0);
+  g_assert_cmpint (json_array_get_length (nice), ==, sysconf(_SC_NPROCESSORS_ONLN));
+  g_assert_cmpint (json_array_get_int_element (nice, 0), >=, 0);
+  g_assert_cmpint (json_array_get_int_element (nice, 0), <=, 1000);
+
+  json_array_unref (samples);
+
+  json_object_unref (res);
+  g_object_unref (channel);
+  json_object_unref (options);
+  g_object_unref (transport);
+}
+
+static void
+test_cpu_temperature (void)
+{
+  MockTransport *transport = mock_transport_new ();
+
+  g_signal_connect (transport, "closed", G_CALLBACK (on_transport_closed), NULL);
+
+  g_autoptr(JsonObject) options = json_obj ("{ 'metrics': [ { 'name': 'cpu.temperature' } ],"
+                                  "  'interval': 1000"
+                                  "}");
+
+  CockpitChannel *channel = g_object_new (cockpit_internal_metrics_get_type (),
+                          "transport", transport,
+                          "id", "1234",
+                          "options", options,
+                          NULL);
+
+  cockpit_channel_prepare (channel);
+
+  g_autoptr(GBytes) msg = NULL;
+  /* receive meta information */
+  while ((msg = mock_transport_pop_channel (transport, "1234")) == NULL)
+    g_main_context_iteration (NULL, TRUE);
+
+  g_autoptr(GError) error = NULL;
+  g_autoptr(JsonObject) res = cockpit_json_parse_bytes (msg, &error);
+  g_assert_no_error (error);
+  g_assert (res != NULL);
+  g_assert (json_object_has_member (res, "metrics"));
+
+  JsonArray *metrics = json_object_get_array_member (res, "metrics");
+  g_assert_cmpint (json_array_get_length (metrics), ==, 1);
+
+  JsonObject *description = json_array_get_object_element (metrics, 0);
+  g_assert (description);
+  g_assert_cmpstr (json_object_get_string_member (description, "name"), ==, "cpu.temperature");
+  g_assert_cmpstr (json_object_get_string_member (description, "units"), ==, "celsius");
+
+  g_autoptr(JsonArray) samples = recv_array (transport);
+  g_assert_cmpint (json_array_get_length (samples), ==, 1);
+  JsonArray *core = json_array_get_array_element (samples, 0);
+  g_assert_cmpint (json_array_get_length (core), ==, 1);
+  JsonArray *temperature = json_array_get_array_element (core, 0);
+
+  // file does not exist in virtual machines, skip value check
+  if (json_array_get_length (temperature) >= 1)
+    {
+      g_assert_cmpint (json_array_get_int_element (temperature, 0), >, 0);
+      g_assert_cmpint (json_array_get_int_element (temperature, 0), <, 150);
+    }
+}
+
 int
 main (int argc,
       char *argv[])
@@ -745,6 +928,10 @@ main (int argc,
   g_test_add_func ("/metrics/not-supported", test_not_supported);
 
   g_test_add_func ("/metrics/deprecated-net-all", test_deprecated_net_all);
+  g_test_add_func ("/metrics/cgroup-memory", test_cgroup);
+
+  g_test_add_func ("/metrics/cpu-cores", test_cpu_cores);
+  g_test_add_func ("/metrics/cpu-temperature", test_cpu_temperature);
 
   return g_test_run ();
 }
