@@ -26,10 +26,10 @@
 #include "cockpitchannelsocket.h"
 #include "cockpitwebservice.h"
 #include "cockpitws.h"
-#include "cockpitcertificate.h"
 
 #include "common/cockpitconf.h"
 #include "common/cockpitjson.h"
+#include "common/cockpitwebcertificate.h"
 #include "common/cockpitwebinject.h"
 
 #include "websocket/websocket.h"
@@ -37,13 +37,34 @@
 #include <json-glib/json-glib.h>
 
 #include <gio/gio.h>
-#include <glib/gi18n.h>
 
 #include <string.h>
 
 /* For overriding during tests */
 const gchar *cockpit_ws_shell_component = "/shell/index.html";
 
+static gchar *
+locate_selfsign_ca (void)
+{
+  g_autofree gchar *cert_path = NULL;
+  gchar *ca_path = NULL;
+  gchar *error = NULL;
+
+  cert_path = cockpit_certificate_locate (true, &error);
+  if (cert_path && g_str_has_suffix (cert_path, "/0-self-signed.cert"))
+    {
+      g_autofree gchar *dir = g_path_get_dirname (cert_path);
+      ca_path = g_build_filename (dir, "0-self-signed-ca.pem", NULL);
+      if (!g_file_test (ca_path, G_FILE_TEST_EXISTS))
+        {
+          g_free (ca_path);
+          ca_path = NULL;
+        }
+    }
+
+  return ca_path;
+}
+
 static void
 on_web_socket_noauth (WebSocketConnection *connection,
                       gpointer data)
@@ -64,15 +85,11 @@ on_web_socket_noauth (WebSocketConnection *connection,
 }
 
 static void
-handle_noauth_socket (GIOStream *io_stream,
-                      const gchar *path,
-                      GHashTable *headers,
-                      GByteArray *input_buffer,
-                      gboolean for_tls_proxy)
+handle_noauth_socket (CockpitWebRequest *request)
 {
   WebSocketConnection *connection;
 
-  connection = cockpit_web_service_create_socket (NULL, path, io_stream, headers, input_buffer, for_tls_proxy);
+  connection = cockpit_web_service_create_socket (NULL, request);
 
   g_signal_connect (connection, "open", G_CALLBACK (on_web_socket_noauth), NULL);
 
@@ -83,14 +100,13 @@ handle_noauth_socket (GIOStream *io_stream,
 /* Called by @server when handling HTTP requests to /cockpit/socket */
 gboolean
 cockpit_handler_socket (CockpitWebServer *server,
-                        const gchar *original_path,
-                        const gchar *path,
-                        const gchar *method,
-                        GIOStream *io_stream,
-                        GHashTable *headers,
-                        GByteArray *input,
+                        CockpitWebRequest *request,
                         CockpitHandlerData *ws)
 {
+  const gchar *path = cockpit_web_request_get_path (request);
+  const gchar *method = cockpit_web_request_get_method (request);
+  GHashTable *headers = cockpit_web_request_get_headers (request);
+
   CockpitWebService *service = NULL;
   const gchar *segment = NULL;
 
@@ -111,18 +127,15 @@ cockpit_handler_socket (CockpitWebServer *server,
   if (g_strcmp0 (method, "GET") != 0)
       return FALSE;
 
-  if (headers)
-    service = cockpit_auth_check_cookie (ws->auth, path, headers);
+  if (headers && ws)
+    service = cockpit_auth_check_cookie (ws->auth, request);
   if (service)
     {
-      cockpit_web_service_socket (service, path, io_stream, headers, input,
-                                  cockpit_web_server_get_flags (server) & COCKPIT_WEB_SERVER_FOR_TLS_PROXY);
-      g_object_unref (service);
+      cockpit_web_service_socket (service, request);
     }
   else
     {
-      handle_noauth_socket (io_stream, path, headers, input,
-                            cockpit_web_server_get_flags (server) & COCKPIT_WEB_SERVER_FOR_TLS_PROXY);
+      handle_noauth_socket (request);
     }
 
   return TRUE;
@@ -130,26 +143,22 @@ cockpit_handler_socket (CockpitWebServer *server,
 
 gboolean
 cockpit_handler_external (CockpitWebServer *server,
-                          const gchar *original_path,
-                          const gchar *path,
-                          const gchar *method,
-                          GIOStream *io_stream,
-                          GHashTable *headers,
-                          GByteArray *input,
+                          CockpitWebRequest *request,
                           CockpitHandlerData *ws)
 {
+  const gchar *path = cockpit_web_request_get_path (request);
+  GHashTable *headers = cockpit_web_request_get_headers (request);
+
   CockpitWebResponse *response = NULL;
   CockpitWebService *service = NULL;
   const gchar *segment = NULL;
   JsonObject *open = NULL;
-  const gchar *query = NULL;
   CockpitCreds *creds;
   const gchar *expected;
   const gchar *upgrade;
   guchar *decoded;
   GBytes *bytes;
   gsize length;
-  gsize seglen;
 
   /* The path must start with /cockpit+xxx/channel/csrftoken? or similar */
   if (path && path[0])
@@ -161,7 +170,7 @@ cockpit_handler_external (CockpitWebServer *server,
   segment += 9;
 
   /* Make sure we are authenticated, otherwise 404 */
-  service = cockpit_auth_check_cookie (ws->auth, path, headers);
+  service = cockpit_auth_check_cookie (ws->auth, request);
   if (!service)
     return FALSE;
 
@@ -171,27 +180,14 @@ cockpit_handler_external (CockpitWebServer *server,
   expected = cockpit_creds_get_csrf_token (creds);
   g_return_val_if_fail (expected != NULL, FALSE);
 
-  /* The end of the token */
-  query = strchr (segment, '?');
-  if (query)
-    {
-      seglen = query - segment;
-      query += 1;
-    }
-  else
-    {
-      seglen = strlen (segment);
-      query = "";
-    }
-
   /* No such path is valid */
-  if (strlen (expected) != seglen || memcmp (expected, segment, seglen) != 0)
+  if (!g_str_equal (segment, expected))
     {
       g_message ("invalid csrf token");
       return FALSE;
     }
 
-  decoded = g_base64_decode (query, &length);
+  decoded = g_base64_decode (cockpit_web_request_get_query (request), &length);
   if (decoded)
     {
       bytes = g_bytes_new_take (decoded, length);
@@ -205,10 +201,7 @@ cockpit_handler_external (CockpitWebServer *server,
 
   if (!open)
     {
-      response = cockpit_web_response_new (io_stream, original_path, path, NULL, headers,
-                                           (cockpit_web_server_get_flags (server) & COCKPIT_WEB_SERVER_FOR_TLS_PROXY) ?
-                                             COCKPIT_WEB_RESPONSE_FOR_TLS_PROXY : COCKPIT_WEB_RESPONSE_NONE);
-
+      response = cockpit_web_request_respond (request);
       cockpit_web_response_error (response, 400, NULL, NULL);
       g_object_unref (response);
     }
@@ -217,17 +210,11 @@ cockpit_handler_external (CockpitWebServer *server,
       upgrade = g_hash_table_lookup (headers, "Upgrade");
       if (upgrade && g_ascii_strcasecmp (upgrade, "websocket") == 0)
         {
-          cockpit_channel_socket_open (service, open, original_path, path, io_stream, headers, input,
-                                       cockpit_web_server_get_flags (server) & COCKPIT_WEB_SERVER_FOR_TLS_PROXY);
+          cockpit_channel_socket_open (service, open, request);
         }
       else
         {
-          response = cockpit_web_response_new (io_stream, original_path, path, NULL, headers,
-                                               (cockpit_web_server_get_flags (server) & COCKPIT_WEB_SERVER_FOR_TLS_PROXY) ?
-                                                 COCKPIT_WEB_RESPONSE_FOR_TLS_PROXY : COCKPIT_WEB_RESPONSE_NONE);
-          cockpit_web_response_set_method (response, method);
-          cockpit_channel_response_open (service, headers, response, open);
-          g_object_unref (response);
+          cockpit_channel_response_open (service, request, open);
         }
       json_object_unref (open);
     }
@@ -259,7 +246,8 @@ add_oauth_to_environment (JsonObject *environment)
 }
 
 static void
-add_page_to_environment (JsonObject *object)
+add_page_to_environment (JsonObject *object,
+                         gboolean    is_cockpit_client)
 {
   static gint page_login_to = -1;
   gboolean require_host = FALSE;
@@ -279,7 +267,7 @@ add_page_to_environment (JsonObject *object)
                                                       G_FILE_TEST_IS_EXECUTABLE));
     }
 
-  require_host = cockpit_conf_bool ("WebService", "RequireHost", FALSE);
+  require_host = is_cockpit_client || cockpit_conf_bool ("WebService", "RequireHost", FALSE);
 
   json_object_set_boolean_member (page, "connect", page_login_to);
   json_object_set_boolean_member (page, "require_host", require_host);
@@ -299,7 +287,7 @@ build_environment (GHashTable *os_release)
    * the corresponding information is not a leak.
    */
   static const gchar *release_fields[] = {
-    "NAME", "ID", "PRETTY_NAME", "VARIANT", "VARIANT_ID", "CPE_NAME", "ID_LIKE"
+    "NAME", "ID", "PRETTY_NAME", "VARIANT", "VARIANT_ID", "CPE_NAME", "ID_LIKE", "DOCUMENTATION_URL"
   };
 
   static const gchar *prefix = "\n    <script>\nvar environment = ";
@@ -310,12 +298,15 @@ build_environment (GHashTable *os_release)
   JsonObject *object;
   const gchar *value;
   gchar *hostname;
-  g_autofree gchar *ca_path;
   JsonObject *osr;
   gint i;
 
   object = json_object_new ();
-  add_page_to_environment (object);
+
+  gboolean is_cockpit_client = cockpit_conf_bool ("WebService", "X-For-CockpitClient", FALSE);
+  json_object_set_boolean_member (object, "is_cockpit_client", is_cockpit_client);
+
+  add_page_to_environment (object, is_cockpit_client);
 
   hostname = g_malloc0 (HOST_NAME_MAX + 1);
   gethostname (hostname, HOST_NAME_MAX);
@@ -337,10 +328,25 @@ build_environment (GHashTable *os_release)
 
   add_oauth_to_environment (object);
 
-  ca_path = cockpit_certificate_locate_selfsign_ca ();
+  g_autofree gchar *ca_path = locate_selfsign_ca ();
   if (ca_path)
     json_object_set_string_member (object, "CACertUrl", "/ca.cer");
 
+  g_autofree gchar *contents = NULL;
+  g_autoptr(GError) error = NULL;
+  gsize len;
+
+  const gchar *banner = cockpit_conf_string ("Session", "Banner");
+  if (banner)
+    {
+      // TODO: parse macros (see `man agetty` for possible macros)
+      g_file_get_contents (banner, &contents, &len, &error);
+      if (error)
+        g_message ("error loading contents of banner: %s", error->message);
+      else
+        json_object_set_string_member (object, "banner", contents);
+    }
+
   bytes = cockpit_json_write_bytes (object);
   json_object_unref (object);
 
@@ -357,6 +363,7 @@ send_login_html (CockpitWebResponse *response,
                  GHashTable *headers)
 {
   static const gchar *marker = "<meta insert_dynamic_content_here>";
+  static const gchar *po_marker = "/*insert_translations_here*/";
 
   CockpitWebFilter *filter;
   GBytes *environment;
@@ -366,6 +373,7 @@ send_login_html (CockpitWebResponse *response,
   GBytes *url_bytes = NULL;
   CockpitWebFilter *filter2 = NULL;
   const gchar *url_root = NULL;
+  const gchar *accept = NULL;
   gchar *content_security_policy = NULL;
   gchar *cookie_line = NULL;
   gchar *base;
@@ -395,16 +403,17 @@ send_login_html (CockpitWebResponse *response,
 
   cockpit_web_response_set_cache_type (response, COCKPIT_WEB_RESPONSE_NO_CACHE);
 
-  if (ws->login_po_html)
+  if (ws->login_po_js)
     {
       language = cockpit_web_server_parse_cookie (headers, "CockpitLang");
       if (!language)
         {
-          languages = cockpit_web_server_parse_languages (headers, NULL);
+          accept = g_hash_table_lookup (headers, "Accept-Language");
+          languages = cockpit_web_server_parse_accept_list (accept, NULL);
           language = languages[0];
         }
 
-      po_bytes = cockpit_web_response_negotiation (ws->login_po_html, NULL, language, NULL, &error);
+      po_bytes = cockpit_web_response_negotiation (ws->login_po_js, NULL, language, NULL, NULL, &error);
       if (error)
         {
           g_message ("%s", error->message);
@@ -412,14 +421,14 @@ send_login_html (CockpitWebResponse *response,
         }
       else if (po_bytes)
         {
-          filter3 = cockpit_web_inject_new (marker, po_bytes, 1);
+          filter3 = cockpit_web_inject_new (po_marker, po_bytes, 1);
           g_bytes_unref (po_bytes);
           cockpit_web_response_add_filter (response, filter3);
           g_object_unref (filter3);
         }
     }
 
-  bytes = cockpit_web_response_negotiation (ws->login_html, NULL, NULL, NULL, &error);
+  bytes = cockpit_web_response_negotiation (ws->login_html, NULL, NULL, NULL, NULL, &error);
   if (error)
     {
       g_message ("%s", error->message);
@@ -433,7 +442,7 @@ send_login_html (CockpitWebResponse *response,
   else
     {
       /* The login Content-Security-Policy allows the page to have inline <script> and <style> tags. */
-      gboolean secure = g_strcmp0 (cockpit_web_response_get_protocol (response, headers), "https") == 0;
+      gboolean secure = g_strcmp0 (cockpit_web_response_get_protocol (response), "https") == 0;
       cookie_line = cockpit_auth_empty_cookie_value (path, secure);
       content_security_policy = cockpit_web_response_security_policy ("default-src 'self' 'unsafe-inline'",
                                                                       cockpit_web_response_get_origin (response));
@@ -520,12 +529,10 @@ on_login_complete (GObject *object,
 static void
 handle_login (CockpitHandlerData *data,
               CockpitWebService *service,
-              const gchar *path,
-              GHashTable *headers,
+              CockpitWebRequest *request,
               CockpitWebResponse *response)
 {
   GHashTable *out_headers;
-  GIOStream *io_stream;
   CockpitCreds *creds;
   JsonObject *creds_json = NULL;
 
@@ -540,9 +547,7 @@ handle_login (CockpitHandlerData *data,
       return;
     }
 
-  io_stream = cockpit_web_response_get_stream (response);
-  cockpit_auth_login_async (data->auth, path,io_stream, headers,
-                            on_login_complete, g_object_ref (response));
+  cockpit_auth_login_async (data->auth, request, on_login_complete, g_object_ref (response));
 }
 
 static void
@@ -621,6 +626,7 @@ handle_shell (CockpitHandlerData *data,
 
 gboolean
 cockpit_handler_default (CockpitWebServer *server,
+                         CockpitWebRequest *request,
                          const gchar *path,
                          GHashTable *headers,
                          CockpitWebResponse *response,
@@ -638,7 +644,7 @@ cockpit_handler_default (CockpitWebServer *server,
              g_str_equal (path, "/cockpit");
 
   // Check for auth
-  service = cockpit_auth_check_cookie (data->auth, path, headers);
+  service = cockpit_auth_check_cookie (data->auth, request);
 
   /* Stuff in /cockpit or /cockpit+xxx */
   if (resource)
@@ -663,7 +669,7 @@ cockpit_handler_default (CockpitWebServer *server,
     {
       if (g_str_equal (remainder, "/login"))
         {
-          handle_login (data, service, path, headers, response);
+          handle_login (data, service, request, response);
         }
       else
         {
@@ -682,6 +688,7 @@ cockpit_handler_default (CockpitWebServer *server,
 
 gboolean
 cockpit_handler_root (CockpitWebServer *server,
+                      CockpitWebRequest *request,
                       const gchar *path,
                       GHashTable *headers,
                       CockpitWebResponse *response,
@@ -694,6 +701,7 @@ cockpit_handler_root (CockpitWebServer *server,
 
 gboolean
 cockpit_handler_ping (CockpitWebServer *server,
+                      CockpitWebRequest *request,
                       const gchar *path,
                       GHashTable *headers,
                       CockpitWebResponse *response,
@@ -729,6 +737,7 @@ cockpit_handler_ping (CockpitWebServer *server,
 
 gboolean
 cockpit_handler_ca_cert (CockpitWebServer *server,
+                         CockpitWebRequest *request,
                          const gchar *path,
                          GHashTable *headers,
                          CockpitWebResponse *response,
@@ -736,7 +745,7 @@ cockpit_handler_ca_cert (CockpitWebServer *server,
 {
   g_autofree gchar *ca_path = NULL;
 
-  ca_path = cockpit_certificate_locate_selfsign_ca ();
+  ca_path = locate_selfsign_ca ();
   if (ca_path == NULL) {
     cockpit_web_response_error (response, 404, NULL, "CA certificate not found");
     return TRUE;
