@@ -27,6 +27,8 @@
 #include "common/cockpitpipe.h"
 #include "common/cockpitconf.h"
 #include "common/cockpitpipetransport.h"
+#include "common/cockpitsystem.h"
+#include "testlib/cockpittest.h"
 #include "common/cockpitwebserver.h"
 #include "common/cockpitwebinject.h"
 
@@ -115,12 +117,26 @@ on_bus_acquired (GDBusConnection *connection,
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean
-mock_http_qs (CockpitWebResponse *response)
+mock_http_info (CockpitWebRequest *request,
+                CockpitWebResponse *response)
+{
+  g_autoptr(JsonObject) info = json_object_new ();
+  json_object_set_boolean_member (info, "pybridge", strstr (bridge_argv[0], "py") != NULL);
+  json_object_set_boolean_member (info, "skip_slow_tests", g_getenv ("COCKPIT_SKIP_SLOW_TESTS") != NULL);
+
+  g_autoptr(GBytes) bytes = cockpit_json_write_bytes (info);
+  cockpit_web_response_content (response, NULL, bytes, NULL);
+  return TRUE;
+}
+
+static gboolean
+mock_http_qs (CockpitWebRequest *request,
+              CockpitWebResponse *response)
 {
   const gchar *qs;
   GBytes *bytes;
 
-  qs = cockpit_web_response_get_query (response);
+  qs = cockpit_web_request_get_query (request);
   if (!qs)
     {
       cockpit_web_response_error (response, 400, NULL, "No query string");
@@ -231,6 +247,32 @@ mock_http_connection (CockpitWebResponse *response)
   return TRUE;
 }
 
+static gboolean
+mock_http_headonly (CockpitWebRequest *request,
+                    CockpitWebResponse *response)
+{
+  if (!g_str_equal (cockpit_web_request_get_method (request), "HEAD"))
+    {
+      cockpit_web_response_error (response, 400, NULL, "Only HEAD allowed on this path");
+    }
+  else
+    {
+      const char *input_data = cockpit_web_request_lookup_header (request, "InputData");
+      if (!input_data)
+        {
+          cockpit_web_response_error (response, 400, NULL, "Requires InputData header");
+          return TRUE;
+        }
+
+      g_autoptr(GHashTable) headers = cockpit_web_server_new_table();
+      g_hash_table_insert (headers, g_strdup ("InputDataLength"), g_strdup_printf ("%zu", strlen (input_data)));
+      cockpit_web_response_headers_full (response, 200, "OK", -1, headers);
+      cockpit_web_response_complete (response);
+    }
+
+  return TRUE;
+}
+
 static gboolean
 mock_http_expect_warnings (CockpitWebResponse *response,
                            GLogLevelFlags warnings)
@@ -245,6 +287,7 @@ mock_http_expect_warnings (CockpitWebResponse *response,
 
 static gboolean
 on_handle_mock (CockpitWebServer *server,
+                CockpitWebRequest *request,
                 const gchar *path,
                 GHashTable *headers,
                 CockpitWebResponse *response,
@@ -253,8 +296,10 @@ on_handle_mock (CockpitWebServer *server,
   g_assert (g_str_has_prefix (path, "/mock/"));
   path += 5;
 
+  if (g_str_equal (path, "/info"))
+    return mock_http_info (request, response);
   if (g_str_equal (path, "/qs"))
-    return mock_http_qs (response);
+    return mock_http_qs (request, response);
   if (g_str_equal (path, "/stream"))
     return mock_http_stream (response);
   if (g_str_equal (path, "/headers"))
@@ -263,6 +308,8 @@ on_handle_mock (CockpitWebServer *server,
     return mock_http_host (response, headers);
   if (g_str_equal (path, "/connection"))
     return mock_http_connection (response);
+  if (g_str_equal (path, "/headonly"))
+    return mock_http_headonly (request, response);
   if (g_str_equal (path, "/expect-warnings"))
     return mock_http_expect_warnings (response, 0);
   if (g_str_equal (path, "/dont-expect-warnings"))
@@ -299,14 +346,11 @@ on_transport_control (CockpitTransport *transport,
 
 static gboolean
 on_handle_stream_socket (CockpitWebServer *server,
-                         const gchar *original_path,
-                         const gchar *path,
-                         const gchar *method,
-                         GIOStream *io_stream,
-                         GHashTable *headers,
-                         GByteArray *input,
+                         CockpitWebRequest *request,
                          gpointer user_data)
 {
+  const gchar *path = cockpit_web_request_get_path (request);
+
   CockpitTransport *transport;
   const gchar *query = NULL;
   CockpitCreds *creds;
@@ -318,7 +362,6 @@ on_handle_stream_socket (CockpitWebServer *server,
   GPid pid = 0;
 
   gchar *value;
-  gchar **env;
   gchar **argv;
 
   if (!g_str_has_prefix (path, "/cockpit/socket"))
@@ -342,17 +385,15 @@ on_handle_stream_socket (CockpitWebServer *server,
       g_clear_object (&bridge);
 
       value = g_strdup_printf ("%d", server_port);
-      env = g_environ_setenv (g_get_environ (), "COCKPIT_TEST_SERVER_PORT", value, TRUE);
 
       argv = g_strdupv (bridge_argv);
       if (query)
         argv[g_strv_length (argv) - 1] = g_strdup (query);
 
-      g_spawn_async_with_pipes (NULL, argv, env,
+      g_spawn_async_with_pipes (NULL, argv, NULL,
                                 G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
                                 NULL, NULL, &pid, &session_stdin, &session_stdout, NULL, &error);
 
-      g_strfreev (env);
       g_free (argv);
       g_free (value);
 
@@ -391,7 +432,7 @@ on_handle_stream_socket (CockpitWebServer *server,
       g_signal_handler_disconnect (transport, handler);
     }
 
-  cockpit_web_service_socket (service, path, io_stream, headers, input, FALSE /* for_tls_proxy */);
+  cockpit_web_service_socket (service, request);
 
   /* Keeps ref on itself until it closes */
   g_object_unref (service);
@@ -427,26 +468,23 @@ on_echo_socket_close (WebSocketConnection *ws,
 
 static gboolean
 on_handle_stream_external (CockpitWebServer *server,
-                           const gchar *original_path,
-                           const gchar *path,
-                           const gchar *method,
-                           GIOStream *io_stream,
-                           GHashTable *headers,
-                           GByteArray *input,
+                           CockpitWebRequest *request,
                            gpointer user_data)
 {
-  CockpitWebResponse *response;
+  const gchar *path = cockpit_web_request_get_path (request);
+  GIOStream *io_stream = cockpit_web_request_get_io_stream (request);
+  GByteArray *input = cockpit_web_request_get_buffer (request);
+  GHashTable *headers = cockpit_web_request_get_headers (request);
+
   gboolean handled = FALSE;
   const gchar *upgrade;
   CockpitCreds *creds;
   const gchar *expected;
-  const gchar *query;
   const gchar *segment;
   JsonObject *open = NULL;
   GBytes *bytes;
   guchar *decoded;
   gsize length;
-  gsize seglen;
 
   if (g_str_has_prefix (path, "/cockpit/echosocket"))
     {
@@ -480,16 +518,10 @@ on_handle_stream_external (CockpitWebServer *server,
       expected = cockpit_creds_get_csrf_token (creds);
       g_return_val_if_fail (expected != NULL, FALSE);
 
-      /* The end of the token */
-      query = strchr (segment, '?');
-      if (!query)
-        query = segment + strlen (segment);
-
       /* No such path is valid */
-      seglen = query - segment;
-      if (strlen(expected) == seglen && memcmp (expected, segment, seglen) == 0)
+      if (g_str_equal (segment, expected))
         {
-          decoded = g_base64_decode (query, &length);
+          decoded = g_base64_decode (cockpit_web_request_get_query (request), &length);
           if (decoded)
             {
               bytes = g_bytes_new_take (decoded, length);
@@ -507,15 +539,12 @@ on_handle_stream_external (CockpitWebServer *server,
           upgrade = g_hash_table_lookup (headers, "Upgrade");
           if (upgrade && g_ascii_strcasecmp (upgrade, "websocket") == 0)
             {
-              cockpit_channel_socket_open (service, open, path, path, io_stream, headers, input, FALSE /* for_tls_proxy */);
+              cockpit_channel_socket_open (service, open, request);
               handled = TRUE;
             }
           else
             {
-              response = cockpit_web_response_new (io_stream, path, path, NULL, headers, COCKPIT_WEB_RESPONSE_NONE);
-              cockpit_web_response_set_method (response, method);
-              cockpit_channel_response_open (service, headers, response, open);
-              g_object_unref (response);
+              cockpit_channel_response_open (service, request, open);
               handled = TRUE;
             }
 
@@ -608,6 +637,7 @@ handle_package_file (CockpitWebServer *server,
 
 static gboolean
 on_handle_resource (CockpitWebServer *server,
+                    CockpitWebRequest *request,
                     const gchar *path,
                     GHashTable *headers,
                     CockpitWebResponse *response,
@@ -633,6 +663,7 @@ on_handle_resource (CockpitWebServer *server,
 
 static gboolean
 on_handle_source (CockpitWebServer *server,
+                  CockpitWebRequest *request,
                   const gchar *path,
                   GHashTable *headers,
                   CockpitWebResponse *response,
@@ -644,7 +675,20 @@ on_handle_source (CockpitWebServer *server,
       inject_address (response, "bus_address", bus_address);
       inject_address (response, "direct_address", direct_address);
     }
-  cockpit_web_response_file (response, path, (const gchar **)server_roots);
+  cockpit_web_response_file_or_gz (response, TRUE, path, (const gchar **)server_roots);
+  return TRUE;
+}
+
+static gboolean
+on_handle_favicon (CockpitWebServer *server,
+                   CockpitWebRequest *request,
+                   const gchar *path,
+                   GHashTable *headers,
+                   CockpitWebResponse *response,
+                   gpointer user_data)
+{
+  const char* roots[] = { SRCDIR "/src/branding/default", NULL };
+  cockpit_web_response_file (response, NULL, roots);
   return TRUE;
 }
 
@@ -655,6 +699,7 @@ server_ready (void)
   GError *error = NULL;
   CockpitWebServer *server;
   gchar *url;
+  const gchar *address_fd_str = g_getenv("TEST_SERVER_ADDRESS_FD");
 
   if (!isatty (1))
     server_port = 0; /* select one automatically */
@@ -662,16 +707,12 @@ server_ready (void)
     server_port = 8765;
 
   server_roots = cockpit_web_response_resolve_roots (roots);
-  server = cockpit_web_server_new (NULL, server_port, /* TCP port to listen to */
-                                   NULL, /* TLS cert */
-                                   COCKPIT_WEB_SERVER_NONE,
-                                   NULL, /* GCancellable* */
-                                   &error);
-  if (server == NULL)
-    {
-      g_critical ("Error setting up web server: %s (%s, %d)",
-                  error->message, g_quark_to_string (error->domain), error->code);
-    }
+  server = cockpit_web_server_new (NULL, /* TLS cert */
+                                   COCKPIT_WEB_SERVER_NONE);
+  server_port = cockpit_web_server_add_inet_listener (server, NULL, server_port, &error);
+  g_assert_no_error (error);
+  g_assert (server_port != 0);
+
 
   g_signal_connect (server, "handle-stream",
                     G_CALLBACK (on_handle_stream_socket), NULL);
@@ -681,15 +722,23 @@ server_ready (void)
                     G_CALLBACK (on_handle_resource), NULL);
   g_signal_connect (server, "handle-resource::/dist/",
                     G_CALLBACK (on_handle_source), NULL);
+  g_signal_connect (server, "handle-resource::/favicon.ico",
+                    G_CALLBACK (on_handle_favicon), NULL);
   g_signal_connect (server, "handle-resource::/mock/",
                     G_CALLBACK (on_handle_mock), NULL);
 
-  server_port = cockpit_web_server_get_port (server);
   url = g_strdup_printf("http://localhost:%d", server_port);
 
   cockpit_web_server_start (server);
 
-  if (!isatty (1))
+  if (address_fd_str)
+    {
+      int fd = atoi (address_fd_str);
+      int r = write (fd, url, strlen (url));
+      close (fd);
+      g_assert (r == strlen (url));
+    }
+  else if (!isatty (1))
     {
       g_print ("%s\n", url);
     }
@@ -763,7 +812,7 @@ setup_path (const char *argv0)
                           old ? ":" : "",
                           old ? old : NULL);
 
-  g_setenv ("PATH", path, TRUE);
+  cockpit_setenv_check ("PATH", path, TRUE);
 
   g_free (path);
   g_free (dir);
@@ -822,15 +871,10 @@ int
 main (int argc,
       char *argv[])
 {
-  GTestDBus *bus;
   GError *error = NULL;
-  GOptionContext *context;
   guint sig_term;
   guint sig_int;
   int i;
-  gchar *guid = NULL;
-  GDBusServer *direct_dbus_server = NULL;
-  gchar *config_dir, *machines_dir;
   gchar *rm_rf_argv[] = {"rm", "-rfv", NULL, NULL};
 
   GOptionEntry entries[] = {
@@ -839,18 +883,18 @@ main (int argc,
 
   signal (SIGPIPE, SIG_IGN);
   /* avoid gvfs (http://bugzilla.gnome.org/show_bug.cgi?id=526454) */
-  g_setenv ("GIO_USE_VFS", "local", TRUE);
+  cockpit_setenv_check ("GIO_USE_VFS", "local", TRUE);
 
   /* playground config directory */
-  config_dir = g_dir_make_tmp ("cockpit.config.XXXXXX", NULL);
+  g_autofree gchar *config_dir = g_dir_make_tmp ("cockpit.config.XXXXXX", NULL);
   g_assert (config_dir);
-  machines_dir = g_build_filename (config_dir, "machines.d", NULL);
-  g_assert (g_mkdir (machines_dir, 0755) == 0);
-  g_free (machines_dir);
+  g_autofree gchar *machines_dir = g_build_filename (config_dir, "cockpit", "machines.d", NULL);
+  g_assert (g_mkdir_with_parents (machines_dir, 0755) == 0);
 
-  g_setenv ("XDG_DATA_HOME", SRCDIR "/src/bridge/mock-resource/home", TRUE);
-  g_setenv ("XDG_DATA_DIRS", SRCDIR "/src/bridge/mock-resource/system", TRUE);
-  g_setenv ("COCKPIT_TEST_CONFIG_DIR", config_dir, TRUE);
+  cockpit_setenv_check ("PYTHONPATH", SRCDIR "/src", TRUE);
+  cockpit_setenv_check ("XDG_DATA_HOME", SRCDIR "/src/bridge/mock-resource/home", TRUE);
+  cockpit_setenv_check ("XDG_DATA_DIRS", SRCDIR "/src/bridge/mock-resource/system", TRUE);
+  cockpit_setenv_check ("XDG_CONFIG_DIRS", config_dir, TRUE);
 
   setup_path (argv[0]);
 
@@ -862,7 +906,7 @@ main (int argc,
   // System cockpit configuration file should not be loaded
   cockpit_config_file = NULL;
 
-  context = g_option_context_new ("- test dbus json server");
+  g_autoptr(GOptionContext) context = g_option_context_new ("- test dbus json server");
   g_option_context_add_main_entries (context, entries, NULL);
   g_option_context_set_ignore_unknown_options (context, TRUE);
   if (!g_option_context_parse (context, &argc, &argv, &error))
@@ -872,17 +916,18 @@ main (int argc,
     }
 
   /* This isolates us from affecting other processes during tests */
-  bus = g_test_dbus_new (G_TEST_DBUS_NONE);
+  g_autoptr(GTestDBus) bus = g_test_dbus_new (G_TEST_DBUS_NONE);
   g_test_dbus_up (bus);
   bus_address = g_test_dbus_get_bus_address (bus);
 
-  guid = g_dbus_generate_guid ();
-  direct_dbus_server = g_dbus_server_new_sync ("unix:tmpdir=/tmp/dbus-tests",
-                                               G_DBUS_SERVER_FLAGS_NONE,
-                                               guid,
-                                               NULL,
-                                               NULL,
-                                               &error);
+  g_autofree gchar *guid = g_dbus_generate_guid ();
+  g_autoptr(GDBusServer) direct_dbus_server = g_dbus_server_new_sync (
+          "unix:tmpdir=/tmp/dbus-tests",
+          G_DBUS_SERVER_FLAGS_NONE,
+          guid,
+          NULL,
+          NULL,
+          &error);
   if (direct_dbus_server == NULL)
     {
       g_printerr ("test-server: %s\n", error->message);
@@ -897,7 +942,10 @@ main (int argc,
   bridge_argv = g_new0 (char *, argc + 2);
   for (i = 0; i < argc; i++)
     bridge_argv[i] = argv[i];
-  bridge_argv[i] = BUILDDIR "/cockpit-bridge";
+
+  /* Default case */
+  if (i == 0)
+    bridge_argv[i] = BUILDDIR "/cockpit-bridge";
 
   // Use a local ssh session command
   cockpit_ws_ssh_program = BUILDDIR "/cockpit-ssh";
@@ -937,21 +985,17 @@ main (int argc,
   g_clear_object (&bridge);
   g_clear_object (&exported);
   g_clear_object (&exported_b);
-  g_clear_object (&direct_dbus_server);
   g_clear_object (&direct);
   g_clear_object (&direct_b);
   g_main_loop_unref (loop);
 
   g_strfreev (server_roots);
   g_test_dbus_down (bus);
-  g_object_unref (bus);
   g_free (bridge_argv);
-  g_free (guid);
 
   /* clean up temporary config dir */
   rm_rf_argv[2] = config_dir;
   g_spawn_sync (NULL, rm_rf_argv, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
-  g_free (config_dir);
 
   return exit_code;
 }
