@@ -2,36 +2,40 @@
  * Fill in module info here.
  */
 
-var info = {
+import path from 'path';
+import fs from 'fs';
+
+import Copy from 'copy-webpack-plugin';
+import Html from 'html-webpack-plugin';
+import MiniCssExtractPlugin from 'mini-css-extract-plugin';
+import CompressionPlugin from 'compression-webpack-plugin';
+import TerserJSPlugin from 'terser-webpack-plugin';
+import CssMinimizerPlugin from 'css-minimizer-webpack-plugin';
+import ESLintPlugin from 'eslint-webpack-plugin';
+import StylelintPlugin from 'stylelint-webpack-plugin';
+import { CockpitPoWebpackPlugin } from './pkg/lib/cockpit-po-plugin.js';
+
+const info = {
     entries: {
+        "base1/cockpit": [
+            "base1/cockpit.js",
+        ],
+
         "apps/apps": [
             "apps/index.jsx"
         ],
 
-        "dashboard/dashboard": [
-            "dashboard/list.js",
-            "dashboard/dashboard.scss",
-        ],
-
-        "docker/docker": [
-            "docker/containers.js"
-        ],
-        "docker/console": [
-            "docker/console.js",
-        ],
-
         "kdump/kdump": [
             "kdump/kdump.js",
-            "kdump/kdump.scss",
         ],
 
-        "machines/machines": [
-            "machines/index.js",
-            "machines/machines.scss",
+        // do *not* call this metrics/metrics -- uBlock origin etc. like to block metrics.{css,js}
+        "metrics/index": [
+            "metrics/index.js",
         ],
 
         "networkmanager/network": [
-            "networkmanager/interfaces.js",
+            "networkmanager/app.jsx",
             "networkmanager/utils.js"
         ],
 
@@ -45,9 +49,6 @@ var info = {
         "playground/exception": [
             "playground/exception.js",
         ],
-        "playground/jquery-patterns": [
-            "playground/jquery-patterns.js",
-        ],
         "playground/metrics": [
             "playground/metrics.js",
         ],
@@ -65,7 +66,6 @@ var info = {
         ],
         "playground/speed": [
             "playground/speed",
-            "playground/speed.css",
         ],
         "playground/test": [
             "playground/test",
@@ -83,23 +83,20 @@ var info = {
             "playground/journal.jsx",
         ],
 
-        "realmd/domain": [
-            "realmd/operation.js",
-        ],
-
         "selinux/selinux": [
             "selinux/setroubleshoot.js",
-            "selinux/setroubleshoot.scss",
         ],
 
         "shell/index": [
             "shell/index.js",
-            "shell/shell.scss",
         ],
 
         "sosreport/sosreport": [
-            "sosreport/index.js",
-            "sosreport/sosreport.scss",
+            "sosreport/index.jsx",
+        ],
+
+        "static/login": [
+            "static/login.js",
         ],
 
         "storaged/storage": [
@@ -108,85 +105,89 @@ var info = {
 
         "systemd/services": [
             "systemd/services/services.jsx",
-            "systemd/services/services.scss",
         ],
         "systemd/logs": [
-            "systemd/logs.js",
-            "systemd/logs.scss",
+            "systemd/logs.jsx",
         ],
         "systemd/overview": [
             "systemd/overview.jsx",
-            "systemd/overview.scss",
         ],
         "systemd/terminal": [
             "systemd/terminal.jsx",
-            "systemd/terminal.scss",
         ],
         "systemd/hwinfo": [
             "systemd/hwinfo.jsx",
-            "systemd/hwinfo.scss",
         ],
-        "systemd/graphs": [
-            "systemd/graphs.js",
-            "systemd/graphs.scss",
-        ],
-
         "tuned/performance": [
-            "tuned/dialog.js",
+            "tuned/dialog.jsx",
         ],
 
         "packagekit/updates": [
             "packagekit/updates.jsx",
-            "packagekit/updates.scss",
         ],
 
         "users/users": [
             "users/local.js",
-            "users/users.scss",
         ]
     },
 
     tests: [
-        "docker/test-docker",
+        "base1/test-base64",
+        "base1/test-browser-storage",
+        "base1/test-cache",
+        "base1/test-chan",
+        "base1/test-dbus-address",
+        "base1/test-dbus-framed",
+        "base1/test-dbus",
+        "base1/test-echo",
+        "base1/test-events",
+        "base1/test-external",
+        "base1/test-file",
+        "base1/test-format",
+        "base1/test-framed-cache",
+        "base1/test-framed",
+        "base1/test-http",
+        "base1/test-journal-renderer",
+        "base1/test-locale",
+        "base1/test-location",
+        "base1/test-metrics",
+        "base1/test-no-jquery",
+        "base1/test-permissions",
+        "base1/test-promise",
+        "base1/test-protocol",
+        "base1/test-series",
+        "base1/test-spawn-proc",
+        "base1/test-spawn",
+        "base1/test-stream",
+        "base1/test-user",
+        "base1/test-utf8",
+        "base1/test-websocket",
 
         "kdump/test-config-client",
 
-        "lib/test-dummy",
-        "lib/test-journal-renderer",
-        "lib/test-machines",
-        "lib/test-patterns",
-
         "networkmanager/test-utils",
 
-        "storaged/test-util",
+        "shell/machines/test-machines",
 
-        "machines/test-machines",
+        "storaged/test-util",
     ],
 
     files: [
         "apps/index.html",
         "apps/default.png",
 
-        "dashboard/index.html",
-
-        "docker/console.html",
-        "docker/index.html",
-        "docker/images/drive-harddisk-symbolic.svg",
-
         "kdump/index.html",
 
-        "machines/index.html",
+        "metrics/index.html",
 
         "networkmanager/index.html",
         "networkmanager/firewall.html",
-        "networkmanager/manifest.json",
 
         "packagekit/index.html",
 
         "playground/index.html",
         "playground/exception.html",
         "playground/hammer.gif",
-        "playground/jquery-patterns.html",
         "playground/metrics.html",
         "playground/pkgs.html",
         "playground/plot.html",
@@ -204,18 +205,21 @@ var info = {
         "shell/images/server-error.png",
         "shell/images/server-large.png",
         "shell/images/server-small.png",
+        "shell/images/cockpit-icon.svg",
+        "shell/images/bg-plain.jpg",
         "shell/index.html",
         "shell/shell.html",
 
         "sosreport/index.html",
         "sosreport/sosreport.png",
 
+        "static/login.html",
+
         "storaged/index.html",
         "storaged/images/storage-array.png",
         "storaged/images/storage-disk.png",
 
         "systemd/index.html",
-        "systemd/graphs.html",
         "systemd/logs.html",
         "systemd/services.html",
         "systemd/terminal.html",
@@ -225,36 +229,27 @@ var info = {
     ]
 };
 
-var externals = {
-    "cockpit": "cockpit",
-    "jquery": "jQuery",
-};
-
 /* ---------------------------------------------------------------------
  * Implementation
  */
 
-var webpack = require("webpack");
-var copy = require("copy-webpack-plugin");
-var html = require('html-webpack-plugin');
-var miniCssExtractPlugin = require('mini-css-extract-plugin');
-var path = require("path");
-var fs = require("fs");
-const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
+process.traceDeprecation = true;
 
 /* These can be overridden, typically from the Makefile.am */
-var srcdir = process.env.SRCDIR || __dirname;
-var builddir = process.env.BUILDDIR || __dirname;
-var distdir = builddir + path.sep + "dist";
-var libdir = path.resolve(srcdir, "pkg" + path.sep + "lib");
-var nodedir = path.resolve(srcdir, "node_modules");
-var section = process.env.ONLYDIR || null;
+const srcdir = process.env.SRCDIR || '.';
+const builddir = process.env.BUILDDIR || '.';
+const libdir = path.resolve(srcdir, "pkg" + path.sep + "lib");
+const nodedir = path.relative(process.cwd(), path.resolve(srcdir, "node_modules"));
+const section = process.env.ONLYDIR || null;
 
 /* A standard nodejs and webpack pattern */
-var production = process.env.NODE_ENV === 'production';
+const production = process.env.NODE_ENV === 'production';
 
-/* development options for faster iteration */
-var eslint = process.env.ESLINT !== '0';
+/* Default to disable eslint for faster production builds */
+const eslint = process.env.ESLINT ? (process.env.ESLINT !== '0') : !production;
+
+/* Default to disable csslint for faster production builds */
+const stylelint = process.env.STYLELINT ? (process.env.STYLELINT !== '0') : !production;
 
 /*
  * Note that we're avoiding the use of path.join as webpack and nodejs
@@ -268,8 +263,8 @@ var eslint = process.env.ESLINT !== '0';
  */
 
 function vpath(/* ... */) {
-    var filename = Array.prototype.join.call(arguments, path.sep);
-    var expanded = builddir + path.sep + filename;
+    const filename = Array.prototype.join.call(arguments, path.sep);
+    let expanded = builddir + path.sep + filename;
     if (fs.existsSync(expanded))
         return expanded;
     expanded = srcdir + path.sep + filename;
@@ -279,122 +274,116 @@ function vpath(/* ... */) {
 }
 
 /* Qualify all the paths in entries */
-Object.keys(info.entries).forEach(function(key) {
+Object.keys(info.entries).forEach(key => {
     if (section && key.indexOf(section) !== 0) {
         delete info.entries[key];
         return;
     }
 
-    info.entries[key] = info.entries[key].map(function(value) {
-        if (value.indexOf("/") === -1)
-            return value;
-        else
-            return vpath("pkg", value);
-    });
+    info.entries[key] = info.entries[key].map(value => (value.indexOf("/") === -1) ? value : vpath("pkg", value));
 });
 
 /* Qualify all the paths in files listed */
-var files = [];
-info.files.forEach(function(value) {
+const files = [];
+info.files.forEach(value => {
     if (!section || value.indexOf(section) === 0)
         files.push({ from: vpath("pkg", value), to: value });
 });
+if (section) {
+    const manifest = section + "manifest.json";
+    files.push({ from: vpath("pkg", manifest), to: manifest });
+}
 info.files = files;
 
-// Hide mini-css-extract-plugin spam logs
-class CleanUpStatsPlugin {
-  shouldPickStatChild(child) {
-    return child.name.indexOf('mini-css-extract-plugin') !== 0;
-  }
+// main font for all our pages
+const redhat_fonts = [
+    "Text-updated-Bold", "Text-updated-BoldItalic", "Text-updated-Italic", "Text-updated-Medium", "Text-updated-MediumItalic", "Text-updated-Regular",
+    "Display-updated-Black", "Display-updated-BlackItalic", "Display-updated-Bold", "Display-updated-BoldItalic",
+    "Display-updated-Italic", "Display-updated-Medium", "Display-updated-MediumItalic", "Display-updated-Regular",
+    "Mono-updated-Bold", "Mono-updated-BoldItalic", "Mono-updated-Italic", "Mono-updated-Medium", "Mono-updated-MediumItalic", "Mono-updated-Regular",
+].map(name => {
+    const subdir = 'RedHat' + name.split('-')[0];
+    const fontsdir = '@patternfly/patternfly/assets/fonts/RedHatFont-updated';
 
-  apply(compiler) {
-    compiler.hooks.done.tap('CleanUpStatsPlugin', (stats) => {
-      const children = stats.compilation.children;
-      if (Array.isArray(children)) {
-        stats.compilation.children = children
-          .filter(child => this.shouldPickStatChild(child));
-      }
-    });
-  }
-}
+    return {
+        // Rename the RedHat*-updated files to not contain the 'updated' string so as to keep compatibility with external plugins
+        // which expect the non-updated font file names in `static` folder
+        from: path.resolve(nodedir, fontsdir, subdir, 'RedHat' + name + '.woff2'),
+        to: 'static/fonts/RedHat' + name.replace("-updated", "") + ".woff2"
+    };
+});
 
-var plugins = [
-    new copy(info.files),
-    new miniCssExtractPlugin("[name].css"),
-    new CleanUpStatsPlugin(),
-    new OptimizeCSSAssetsPlugin({cssProcessorOptions: {map: {inline: false} } }),
+const plugins = [
+    new Copy({ patterns: info.files }),
+    new MiniCssExtractPlugin({ filename: "[name].css" }),
+    new CockpitPoWebpackPlugin({
+        subdir: section,
+        // login page does not have cockpit.js, but reads window.cockpit_po
+        wrapper: (section === 'static/') ? 'window.cockpit_po = PO_DATA;' : undefined,
+    }),
 ];
 
-var output = {
-    path: distdir,
-    filename: "[name].js",
-    sourceMapFilename: "[file].map",
-};
-
-/* Only minimize when in production mode */
 if (production) {
-    /* Rename output files when minimizing */
-    output.filename = "[name].min.js";
+    plugins.push(new CompressionPlugin({
+        test: /\.(css|html|js|txt)$/,
+        deleteOriginalAssets: true,
+        exclude: [
+            '/test-[^/]+.$', // don't compress test cases
+            '^static/[^/]+$', // cockpit-ws cannot currently serve compressed login page
+            '\\.html$', // HTML pages get patched by cockpit-ws, can't be compressed
+        ].map(r => new RegExp(r)),
+    }));
 }
 
+if (eslint) {
+    plugins.push(new ESLintPlugin({ extensions: ["js", "jsx"] }));
+}
+
+if (stylelint) {
+    plugins.push(new StylelintPlugin({
+        context: "pkg/" + section,
+    }));
+}
+
+if (section.startsWith('static'))
+    plugins.push(new Copy({ patterns: redhat_fonts }));
+
 /* Fill in the tests properly */
-info.tests.forEach(function(test) {
-    var ext = production ? ".min.js" : ".js";
+info.tests.forEach(test => {
     if (!section || test.indexOf(section) === 0) {
         info.entries[test] = vpath("pkg", test + ".js");
-        plugins.push(new html({
+        plugins.push(new Html({
             title: path.basename(test),
             filename: test + ".html",
             template: libdir + path.sep + "qunit-template.html",
-            builddir: test.split("/").map(function() { return "../" }).join(""),
-            script: path.basename(test + ext),
+            builddir: test.split("/").map(() => "../").join(""),
+            script: path.basename(test + '.js'),
             inject: false,
         }));
     }
 });
 
-var aliases = {
-    "d3": "d3/d3.js",
-    "moment": "moment/moment.js",
+const aliases = {
     "font-awesome": path.resolve(nodedir, 'font-awesome-sass/assets/stylesheets'),
 };
 
-/* HACK: To get around redux warning about reminimizing code */
-if (production)
-    aliases["redux/dist/redux"] = "redux/dist/redux.min.js";
-
-
-var babel_loader = {
-    loader: "babel-loader",
-    options: {
-        presets: [
-            ["@babel/env", {
-                "targets": {
-                    "chrome": "57",
-                    "firefox": "52",
-                    "safari": "10.3",
-                    "edge": "16",
-                    "opera": "44"
-                }
-            }],
-            "@babel/preset-react"
-        ]
-    }
-}
-
-module.exports = {
+export default {
     mode: production ? 'production' : 'development',
     resolve: {
         alias: aliases,
-        modules: [ libdir, nodedir ],
+        modules: [libdir, nodedir],
         extensions: ["*", ".js", ".json"]
     },
+    resolveLoader: {
+        modules: [nodedir, './pkg/lib'],
+    },
     entry: info.entries,
-    output: output,
-    externals: externals,
+    // cockpit.js gets included via <script>, everything else should be bundled
+    externals: (section === 'kdump/' || section === 'base1/') ? {} : { cockpit: "cockpit" },
     plugins: plugins,
 
-    devtool: "source-map",
+    devtool: production ? false : "source-map",
+    stats: "errors-warnings",
 
     // disable noisy warnings about exceeding the recommended size limit
     performance: {
@@ -402,14 +391,24 @@ module.exports = {
         maxEntrypointSize: 20000000,
     },
 
+    watchOptions: {
+        ignored: /node_modules/
+    },
+
+    optimization: {
+        minimize: production,
+        minimizer: [
+            new TerserJSPlugin(),
+            new CssMinimizerPlugin({
+                minimizerOptions: {
+                    preset: ['lite']
+                }
+            })
+        ],
+    },
+
     module: {
         rules: [
-            {
-                enforce: 'pre',
-                test: eslint ? /\.(js|jsx)$/ : /dont.match.me/,
-                exclude: /\/node_modules\/.*\//, // exclude external dependencies
-                loader: "eslint-loader"
-            },
             {
                 test: /\.js$/,
                 exclude: /\/node_modules\/.*\//, // exclude external dependencies
@@ -417,24 +416,24 @@ module.exports = {
             },
             /* these modules need to be babel'ed, they cause bugs in their dist'ed form */
             {
-                test: /\/node_modules\/.*(@novnc|react-table).*\.js$/,
-                use: babel_loader
+                test: /\/node_modules\/.*(react-table).*\.js$/,
+                use: "babel-loader"
             },
             {
                 test: /\.(js|jsx)$/,
                 // exclude external dependencies; it's too slow, and they are already plain JS except the above
-                exclude: /\/node_modules\/.*\//,
-                use: babel_loader
+                // also exclude unit tests, we don't need it for them, just a waste and makes failures harder to read
+                exclude: /\/node_modules|\/test-[^/]*\.js/,
+                use: "babel-loader"
             },
-            /* HACK: remove unwanted fonts from PatternFly's css */
             {
-                test: /patternfly-cockpit.scss$/,
+                test: /patternfly-4-cockpit.scss$/,
                 use: [
-                    miniCssExtractPlugin.loader,
+                    MiniCssExtractPlugin.loader,
                     {
                         loader: 'css-loader',
                         options: {
-                            sourceMap: true,
+                            sourceMap: !production,
                             url: false,
                         },
                     },
@@ -443,20 +442,8 @@ module.exports = {
                         options: {
                             multiple: [
                                 {
-                                    search: /src:url[(]"patternfly-icons-fake-path\/glyphicons-halflings-regular[^}]*/g,
-                                    replace: 'font-display:block; src:url("../base1/fonts/glyphicons.woff") format("woff");',
-                                },
-                                {
-                                    search: /src:url[(]"patternfly-fonts-fake-path\/PatternFlyIcons[^}]*/g,
-                                    replace: 'src:url("../base1/fonts/patternfly.woff") format("woff");',
-                                },
-                                {
-                                    search: /src:url[(]"patternfly-fonts-fake-path\/fontawesome[^}]*/,
-                                    replace: 'font-display:block; src:url("../base1/fonts/fontawesome.woff?v=4.2.0") format("woff");',
-                                },
-                                {
-                                    search: /src:url\("patternfly-icons-fake-path\/pficon[^}]*/g,
-                                    replace: 'src:url("../base1/fonts/patternfly.woff") format("woff");',
+                                    search: /src: ?url\("patternfly-icons-fake-path\/pficon[^}]*/g,
+                                    replace: "src:url('fonts/patternfly.woff')format('woff');",
                                 },
                                 {
                                     search: /@font-face[^}]*patternfly-fonts-fake-path[^}]*}/g,
@@ -468,52 +455,36 @@ module.exports = {
                     {
                         loader: 'sass-loader',
                         options: {
+                            sourceMap: !production,
                             sassOptions: {
-                                includePaths: [
-                                    // Teach webpack to resolve these references in order to build PF3 scss
-                                    path.resolve(nodedir),
-                                    path.resolve(nodedir, 'font-awesome-sass', 'assets', 'stylesheets'),
-                                    path.resolve(nodedir, 'patternfly', 'dist', 'sass'),
-                                    path.resolve(nodedir, 'bootstrap-sass', 'assets', 'stylesheets'),
-                                ],
-                                outputStyle: 'compressed',
+                                quietDeps: true,
+                                outputStyle: production ? 'compressed' : undefined,
                             },
-                            sourceMap: true,
                         },
                     },
                 ]
             },
             {
                 test: /\.s?css$/,
-                exclude: /patternfly-cockpit.scss/,
+                exclude: /patternfly-(4-)?cockpit.scss/,
                 use: [
-                    miniCssExtractPlugin.loader,
+                    MiniCssExtractPlugin.loader,
                     {
                         loader: 'css-loader',
                         options: {
-                            sourceMap: true,
+                            sourceMap: !production,
                             url: false
                         }
                     },
                     {
                         loader: 'sass-loader',
                         options: {
-                            sourceMap: true,
+                            sourceMap: !production,
                             sassOptions: {
-                                outputStyle: 'compressed',
-                            }
-                        }
-                    },
-                    {
-                        loader: 'sass-resources-loader',
-                            // Make PF3 and PF4 variables globably accessible to be used by the components scss
-                            options: {
-                                resources: [
-                                    path.resolve(libdir, './_global-variables.scss'),
-                                    path.resolve(nodedir, './@patternfly/patternfly/patternfly-variables.scss'),
-                                    path.resolve(nodedir, './patternfly/dist/sass/patternfly/_variables.scss')
-                                ],
+                                quietDeps: true,
+                                outputStyle: production ? 'compressed' : undefined,
                             },
+                        },
                     },
                 ]
             },
